#include "stdafx.h"
#include "../Thread/ThreadPool.h"
#include "../FileData.h"
#include "Archive.h"
namespace Themis {

CArchive::CArchive(const CFileName& oTmpBaseDir, const ArchiveOwner& owner)
: m_nParentIndex(-1), extractAllFlag(0), owner(owner) {
    m_pTempDir.reset(new CTempDir(oTmpBaseDir));
}

CArchive::CArchive(const CFileName& oTmpBaseDir, const std::tr1::weak_ptr<CArchive>& pParent, int nFileIndex, const ArchiveOwner& owner)
: m_pParent(pParent), m_nParentIndex(nFileIndex), extractAllFlag(0), owner(owner) {
    m_pTempDir.reset(new CTempDir(oTmpBaseDir));
}

CArchive::~CArchive() {
    extractAllFlag = -1; // 𓀂Ă璆f
    // qIuWFNgɍ폜
    m_vctChildren.clear();
}

bool CArchive::Open(const CFileName& oFileName) {
    try {
        m_pArchiver.reset(new CArchiver);
        m_pArchiver->Open(oFileName);
        m_pArchiver->SetQueryPass(owner.queryPass);
        // 𓀃tO𖢉𓀂ɐݒ
        ResetExtractFlags();
        return true;
    } catch (std::exception&) {
        m_pArchiver.reset();
        return false;
    }
}

/// ɉ𓀃tÕZbg
void CArchive::ResetExtractFlags() {
    extractAllFlag = 0;
    m_vctExtractFlag.resize(m_pArchiver->GetFileCount());
    std::fill(m_vctExtractFlag.begin(), m_vctExtractFlag.end(), 0);
}

void CArchive::GetFileList(vector<CArchive::SFileInfo>& rList) {
    Themis_DebugAssert(IsSupported());

    rList.reserve(rList.size() + m_pArchiver->GetFileCount());

    vector<string> suffixes = CArchiver::GetSupportedSuffixesLc();
    for (int i = 0, n = m_pArchiver->GetFileCount(); i < n; i++) {
        if (std::find(suffixes.begin(), suffixes.end(),
            CFileName(m_pArchiver->GetFileName(i)).GetSuffixLc()) != suffixes.end()) {
            // gqɂۂΉ𓀂Ċm߂Ă݂
            InnerExtract(i);
            std::tr1::shared_ptr<CArchive> p(new CArchive(
                m_pTempDir->GetBaseDir(), shared_from_this(), i, owner));
            if (p->Open(InnerGetExistFileName(i))) {
                p->GetFileList(rList);
                m_vctChildren.push_back(p);
            } else {
                // ΉĂ鏑ɂȂۂ̂ŕʂɃXg֒ǉ
                CArchive::SFileInfo oInfo;
                oInfo.strViewName = InnerGetViewFileName(i);
                oInfo.pArchive = shared_from_this();
                oInfo.nArchiveIndex = i;
                rList.push_back(oInfo);
            }
        } else {
            // ʂ̃t@Cۂ̂ŃXg֒ǉ
            CArchive::SFileInfo oInfo;
            oInfo.strViewName = InnerGetViewFileName(i);
            oInfo.pArchive = shared_from_this();
            oInfo.nArchiveIndex = i;
            rList.push_back(oInfo);
        }
    }
}

pair<std::tr1::weak_ptr<CArchive>, int> CArchive::Find(const string& strViewName) {
    Themis_DebugAssert(IsSupported());
    // ̂ƂT
    int n = InnerFindFile(strViewName);
    if (0 <= n) {
        return pair<std::tr1::weak_ptr<CArchive>, int>(shared_from_this(), n);
    } else {
        //qɂɒT
        for (size_t i = 0; i < m_vctChildren.size(); i++) {
            int n = m_vctChildren[i]->InnerFindFile(strViewName);
            if (0 <= n) {
                return pair<std::tr1::weak_ptr<CArchive>, int>(m_vctChildren[i], n);
            }
        }
    }
    // ̏ɓ̃t@CȂۂ
    return pair<std::tr1::weak_ptr<CArchive>, int>(std::tr1::weak_ptr<CArchive>(), -1);
}

size_t CArchive::GetFileSize(int i) {
    Themis_DebugAssert(IsSupported());
    return m_pArchiver->GetFileSize(i);
}

FILETIME CArchive::GetLastUpdate(int i) {
    Themis_DebugAssert(IsSupported());
    return m_pArchiver->GetLastUpdate(i);
}

std::tr1::shared_ptr<CFileData> CArchive::LoadFile(int i) {
    Themis_DebugAssert(IsSupported());
    return InnerLoadFile(i);
}

bool CArchive::IsSupportedFileName(const CFileName& oFileName) {
    vector<string> suffixes = CArchiver::GetSupportedSuffixesLc();
    return std::find(suffixes.begin(), suffixes.end(), oFileName.GetSuffixLc()) != suffixes.end();
}

/// t@CJĂ
std::tr1::shared_ptr<CFileData> CArchive::InnerLoadFile(int nFileIndex) {
    std::tr1::shared_ptr<CFileData> p;
    if (m_pArchiver->GetType() == IArchiver::eTypeLoadable) {
        CFileName oFileName(InnerGetViewFileName(nFileIndex));
        p.reset(new CFileData(oFileName));
        m_pArchiver->LoadFile(nFileIndex, p->GetMemory());
    } else {
        InnerExtract(nFileIndex);
        p.reset(new CFileData(InnerGetExistFileName(nFileIndex)));
        CFile(p->GetName()).Load(p->GetMemory());
    }
    return p;
}

/// ViewName珑ɓ̏擾B-1
int CArchive::InnerFindFile(const string& strViewName) const {
    string strArcName = GetViewName();
    // Ƀt@C̔r
    if (strViewName.StartsWith(strArcName)) {
        // ɕ\ȍ~̕
        string s = CFileName::SlashToYen(strViewName.Substring(strArcName.GetLength()));
        for (int i = 0, n = m_pArchiver->GetFileCount(); i < n; i++) {
            if (s == CFileName::SlashToYen(m_pArchiver->GetFileName(i))) {
                return i;
            }
        }
    }
    return -1;
}

/// w菘̃t@C𓀂
void CArchive::InnerExtract(int nFileIndex) {
    if (extractAllFlag < 0) {
        // 𓀒fς݂莸s
     // throw RuntimeException(L"𓀂Ɏs: " + GetViewName() + m_pArchiver->GetFileName(nFileIndex));
    } else if (extractAllFlag == 2 || m_vctExtractFlag[nFileIndex] == 2) {
        // 𓀍ς݂Ȃ牽ȂB
    } else {
        Themis_DebugAssert(extractAllFlag != 1);
        // \bhɂȂS𓀂Ă܂B
        if (m_pArchiver->GetType() == IArchiver::eTypeSolid) {
            extractAllFlag = 1; // 𓀒Ƃ
            try {
                m_pArchiver->ExtractAll(m_pTempDir->GetName(), true);
                extractAllFlag = 2;
            } catch (...) {
                extractAllFlag = -2;
            }
        } else {
            // t@C̉𓀁B
            try {
                m_pTempDir->Create();
                m_pArchiver->ExtractFile(nFileIndex, m_pTempDir->GetName());
            } catch (std::exception&) {
                m_vctExtractFlag[nFileIndex] = 0;   //𓀎sH
                throw;
            }
            m_vctExtractFlag[nFileIndex] = 2; // 𓀊
        }
    }
}

}
