#include "stdafx.h"
#include "Config.h"
#include "File.h"
#include "ProcessExclusion.h"
#include "Windows/System.h"
#include "String/StringBuilder.h"
#include "String/Encoding.h"
#include "Utility/StringUtility.h"
namespace Themis {

CConfig::CConfig() {
}

CConfig::~CConfig() {
}

void CConfig::Clear() {
    m_oDataList.clear();
}

void CConfig::Delete(const string& strName) {
    m_oDataList.erase(strName);
}

bool CConfig::Exists(const string& strName) const {
    return (m_oDataList.find(strName) != m_oDataList.end());
}

void CConfig::ChanegName(const string& strName1, const string& strName2) {
    if (Exists(strName1)) {
        Indirect(strName2, Indirect(strName1));
        Delete(strName1);
    }
}

/// ږ̗
vector<string> CConfig::GetNames() const {
    vector<string> vct(m_oDataList.size());
    int i = 0;
    for (std::map<string, string>::const_iterator it = m_oDataList.begin(), itEnd = m_oDataList.end(); it != itEnd; ++it) {
        vct[i++] = (*it).first;
    }
    return vct;
}

/// ږEf[^̗
vector<pair<string, string> > CConfig::GetValues() const {
    vector<pair<string, string> > vct(m_oDataList.size());
    int i = 0;
    for (std::map<string, string>::const_iterator it = m_oDataList.begin(), itEnd = m_oDataList.end(); it != itEnd; ++it) {
        vct[i++] = *it;
    }
    return vct;
}

//////////////////////////////////////////////////////////////////////

void CConfig::Str(const string& strName, const string& strValue) {
    Indirect(strName, FormatStrValue(strValue));
}
void CConfig::Num(const string& strName, int nValue) {
    Indirect(strName, ItoA(nValue));
}
void CConfig::Float(const string& strName, boost::call_traits<double>::param_type dValue) {
    Indirect(strName, FtoA(dValue));
}
void CConfig::Bool(const string& strName, bool b) {
    Indirect(strName, b ? L"true" : L"false");
}
void CConfig::Bin(const string& strName, const pair<const void*, size_t>& p) {
    Indirect(strName, FormatBinValue(p.first, p.second));
}
void CConfig::Bin(const string& strName, const vector<uint8_t>& r) {
    Indirect(strName, r.empty() ? string() : FormatBinValue(&r[0], r.size()));
}
void CConfig::Point(const string& strName, const CPoint& pt) {
    Indirect(strName, string::Format(L"%d, %d", pt.x, pt.y));
}
void CConfig::Size(const string& strName, const CSize& sz) {
    Indirect(strName, string::Format(L"%d, %d", sz.cx, sz.cy));
}
void CConfig::Rect(const string& strName, const CRect& rc) {
    Indirect(strName, string::Format(L"%d, %d, %d, %d", rc.left, rc.top, rc.GetWidth(), rc.GetHeight()));
}
void CConfig::Color(const string& strName, COLORREF rgb) {
    Indirect(strName, string::Format(L"%d, %d, %d", GetRValue(rgb), GetGValue(rgb), GetBValue(rgb)));
}
void CConfig::File(const string& strName, const string& strValue) {
 // Indirect(strName, FormatStrValue(strValue.Replace(L"\\", L"/")));
    Indirect(strName, strValue);
}
void CConfig::Indirect(const string& strName, const string& strValue) {
    Themis_DebugAssert(!strName.Contains(L" ="));
    m_oDataList[strName] = strValue;
}

//////////////////////////////////////////////////////////////////////

string CConfig::Str(const string& strName) const {
    std::map<string, string>::const_iterator itr = Find(strName);
    if (itr != GetEnd()) {
        return GetStrValue(itr->second);
    }
    throw RuntimeException(L"RtBOf[^ '" + strName + L"' " + L"()" + L"̓ǂݍ݂Ɏs܂");
}

int CConfig::Num(const string& strName) const {
    std::map<string, string>::const_iterator itr = Find(strName);
    if (itr != GetEnd() && !string::IsNullOrEmpty(itr->second)) {
        return AtoI(itr->second);
    }
    throw RuntimeException(L"RtBOf[^ '" + strName + L"' " + L"(l)" + L"̓ǂݍ݂Ɏs܂");
}

double CConfig::Float(const string& strName) const {
    std::map<string, string>::const_iterator itr = Find(strName);
    if (itr != GetEnd() && !string::IsNullOrEmpty(itr->second)) {
        return AtoF(itr->second);
    }
    throw RuntimeException(L"RtBOf[^ '" + strName + L"' " + L"(l)" + L"̓ǂݍ݂Ɏs܂");
}

bool CConfig::Bool(const string& strName) const {
    std::map<string, string>::const_iterator itr = Find(strName);
    if (itr != GetEnd()) {
        if (itr->second == L"true") return true;
        if (itr->second == L"false") return false;
    }
    throw RuntimeException(L"RtBOf[^ '" + strName + L"' " + L"(^Ul)" + L"̓ǂݍ݂Ɏs܂");
}

vector<uint8_t> CConfig::Bin(const string& strName) const {
    std::map<string, string>::const_iterator itr = Find(strName);
    if (itr != GetEnd()) {
        vector<uint8_t> oMemory(((*itr).second.GetLength() + 1) / 2);
        if (!oMemory.empty()) {
            GetBinValue(itr->second, &oMemory[0]);
        }
        return oMemory;
    }
    throw RuntimeException(L"RtBOf[^ '" + strName + L"' " + L"(oCi)" + L"̓ǂݍ݂Ɏs܂");
}

CPoint CConfig::Point(const string& strName) const {
    int x, y;
    if (sscanf(Encoding::GetDefault().GetBytes(Indirect(strName)).c_str(), "%d, %d", &x, &y) == 2) {
        return CPoint(x, y);
    }
    throw RuntimeException(L"RtBOf[^ '" + strName + L"' " + L"(Wl)" + L"̓ǂݍ݂Ɏs܂");
}

CSize CConfig::Size(const string& strName) const {
    int x, y;
    if (sscanf(Encoding::GetDefault().GetBytes(Indirect(strName)).c_str(), "%d, %d", &x, &y) == 2) {
        return CPoint(x, y);
    }
    throw RuntimeException(L"RtBOf[^ '" + strName + L"' " + L"(cl)" + L"̓ǂݍ݂Ɏs܂");
}

CRect CConfig::Rect(const string& strName) const {
    int x, y, z, a;
    if (sscanf(Encoding::GetDefault().GetBytes(Indirect(strName)).c_str(), "%d, %d, %d, %d", &x, &y, &z, &a) == 4) {
        return CRect(x, y, z, a);
    }
    throw RuntimeException(L"RtBOf[^ '" + strName + L"' " + L"(`Wl)" + L"̓ǂݍ݂Ɏs܂");
}
COLORREF CConfig::Color(const string& strName) const {
    int x, y, z;
    if (sscanf(Encoding::GetDefault().GetBytes(Indirect(strName)).c_str(), "%d, %d, %d", &x, &y, &z) == 3) {
        return RGB(x, y, z);
    }
    throw RuntimeException(L"RtBOf[^ '" + strName + L"' " + L"(RGBl)" + L"̓ǂݍ݂Ɏs܂");
}

string CConfig::Indirect(const string& strName) const {
    std::map<string, string>::const_iterator itr = m_oDataList.find(strName);
    if (itr != m_oDataList.end()) {
        return (*itr).second;
    }
 // return string();
    throw RuntimeException(L"RtBOf[^ '" + strName + L"' " + L"̓ǂݍ݂Ɏs܂");
}

//////////////////////////////////////////////////////////////////////

string CConfig::FormatStrValue(const string& strValue) const {
    StringBuilder builder;
    for (int i = 0, nLen = strValue.GetLength(); i < nLen; ) {
        wchar_t c = strValue[i++];
        if      (c == L'\\') { builder.Append(L"\\\\"); }
        else if (c == L'\r') { builder.Append(L"\\r"); }
        else if (c == L'\n') { builder.Append(L"\\n"); }
        else if (c == L'\0') { builder.Append(L"\\0"); }
        else {
            builder.Append(c);
        }
    }
    return builder.ToString();
}

string CConfig::FormatBinValue(const void* p, size_t nSize) const {
    StringBuilder builder;
    const uint8_t* pByte = static_cast<const uint8_t*>(p);
    for (size_t i=0; i < nSize; i++) {
        builder.Append(L"0123456789abcdef"[(pByte[i] >> 4) & 0x0F]);
        builder.Append(L"0123456789abcdef"[pByte[i] & 0x0F]);
    }
    return builder.ToString();
}

string CConfig::GetStrValue(const string& strValue) const {
    StringBuilder builder;
    for (int i = 0, nLen = strValue.GetLength(); i < nLen; ) {
        wchar_t c = strValue[i++];
        if (c == L'\\') {
            wchar_t c2 = strValue[i++];
            if      (c2 == L'\\') { builder.Append(L'\\'); }
            else if (c2 == L'r')  { builder.Append(L'\r'); }
            else if (c2 == L'n')  { builder.Append(L'\n'); }
            else if (c2 == L'0')  { builder.Append(L'\0'); }
            else                  { builder.Append(c); i--; }  //͖
        } else {
            builder.Append(c);
        }
    }
    return builder.ToString();
}
void CConfig::GetBinValue(const string& str, void* p) const {
    uint8_t abyTbl[UCHAR_MAX] = {0};
    int c;
    for (c = L'0'; c <= L'9'; c++) abyTbl[c] = static_cast<uint8_t>(c - L'0');
    for (c = L'a'; c <= L'f'; c++) abyTbl[c] = static_cast<uint8_t>(c - L'a' + 0x0a);
    for (c = L'A'; c <= L'F'; c++) abyTbl[c] = static_cast<uint8_t>(c - L'A' + 0x0A);
    for (int i = 0, nSize = str.GetLength() / 2; i < nSize; i++) {
        static_cast<uint8_t*>(p)[i] = static_cast<uint8_t>(
            ((abyTbl[str[i * 2 + 0]]) << 4) | (abyTbl[str[i * 2 + 1]]));
    }
}

CFileName CConfig::GetDefaultFileName() {
    return CFileName(CSystem::GetModuleFileName()).SetSuffix(L"cfg");
}

bool CConfig::Save() const {
    return Save(GetDefaultFileName());
}

bool CConfig::Load() {
    return Load(GetDefaultFileName());
}

bool CConfig::Save(const CFileName& oFileName) const {
    try {
        CProcessExclusionLock lock(oFileName.Get());

        CFile oFile(CFileName(oFileName.Get() + L".tmp"));
        oFile.Save(SaveToStr());
        oFile.CopyOverwrite(oFileName);
        oFile.Delete();
        return true;
    } catch (std::exception&) {
        Themis_DebugWarning(L"RtBOt@Cos");
    }
    return false;
}

string CConfig::SaveToStr() const {
    StringBuilder builder;
    std::map<string, string>::const_iterator itr = GetBegin();
    std::map<string, string>::const_iterator itrEnd = GetEnd();
    for (; itr!=itrEnd; itr++) {
        builder.Append(GetStorageStr((*itr).first, (*itr).second));
    }
    return builder.ToString();
}

bool CConfig::Load(const CFileName& oFileName) {
    try {
        CProcessExclusionLock lock(oFileName.Get());

        CFile file(oFileName);
        if (!file.Exists()) {
            return false;
        }
        LoadFromStr(file.LoadStr());
        return true;
    } catch (std::exception&) {
        Themis_DebugLog(L"RtBOt@Cǂݍݎs: " + oFileName.Get());
    }
    return false;
}

void CConfig::LoadFromStr(const string& str) {
    List<string> lines = str.Split(Array<string>(L"\r\n", L"\n", L"\r"));
    for (int i = 0, n = lines.GetCount(); i < n; i++) {
        SetStorageStr(lines[i]);
    }
}

string CConfig::GetStorageStr(const string& strName, const string& strValue) const {
    return strName + L" = " + strValue + L"\r\n";
}

bool CConfig::SetStorageStr(const string& strStorage) {
    int n = strStorage.IndexOf(L" = ");
    if (0 <= n) {
        Indirect(strStorage.Substring(0, n), strStorage.Substring(n + 3));
        return true;
    }
    return false;
}

}
