#include "stdafx.h"
#include "Thread/XTime.h"
#include "Thread/Thread.h"
#include "Thread/ThreadPool.h"
#include "Windows/System.h"
#include "Window.h"
#include "CursorHider.h"
namespace Themis {

CCursorHider::CCursorHider(CWindow& rWindow) : m_rWindow(rWindow) {
    m_ptCursor = CPoint(-1, -1);
    m_nTimeout = 1000;
    m_bHide = false;

    threadValid = false;
}

CCursorHider::~CCursorHider() {
    SetEnable(false);
    InnerShow();
}

bool CCursorHider::IsEnable() const {
    return threadValid;
}

void CCursorHider::SetEnable(bool b) {
    if (b != threadValid) {
        if (b) {
            boost::recursive_mutex::scoped_lock lock(mutex);
            threadValid = true;
            thread = ThreadPool::CreateWorkerThread();
            thread->SetWork(std::tr1::bind(&CCursorHider::OnThread, this));
        } else {
            {
                boost::recursive_mutex::scoped_lock lock(mutex);
                threadValid = false;
                condition.notify_all();
            }
            thread->Join();
            if (!m_bForceHide) {
                InnerShow();
            }
        }
    }
}

int CCursorHider::GetTimeout() const {
    return m_nTimeout;
}

void CCursorHider::SetTimeout(int nMilliSec) {
    m_nTimeout = nMilliSec;
}

/// J[\ꂽȂɌĂ
void CCursorHider::Reset() {
    boost::recursive_mutex::scoped_lock lock(mutex);
    condition.notify_all();
}

/// \Ԃɂ
void CCursorHider::ForceHide() {
    boost::recursive_mutex::scoped_lock lock(mutex);
    m_bForceHide = true;
    InnerHide();
    condition.notify_all();
}

/// \Ԃ
void CCursorHider::UnforceHide() {
    boost::recursive_mutex::scoped_lock lock(mutex);
    m_bForceHide = false;
    InnerShow();
    condition.notify_all();
}

void CCursorHider::OnThread() {
    Thread::SetName(L"CursorHiderThread");

    CPoint lastPos = CSystem::GetCursorPos();
    boost::xtime sleepTime = XTime::AfterMilliSeconds(m_nTimeout);

    boost::recursive_mutex::scoped_lock lock(mutex);
    while (threadValid) {
        if (m_bForceHide) {
            // ԕς܂ő҂
            condition.wait(lock);
        } else {
            condition.timed_wait(lock, sleepTime);
            if (!threadValid) return;
            CPoint currentPos = CSystem::GetCursorPos();
            if (lastPos == currentPos && IsWindowValid() && m_rWindow.GetClientRect().IsIn(lastPos)) {
                // ^CAEĝŉBB
                InnerHide();
                // ԕς܂ő҂
             // condition.wait(lock);
                condition.timed_wait(lock, XTime::AfterSeconds(1));
            } else {
                InnerShow();
                // ʒuς肵ۂ̂Ń^CAEgZbg
                lastPos = currentPos;
                sleepTime = XTime::AfterMilliSeconds(m_nTimeout);
            }
        }
    }
}

bool CCursorHider::IsWindowValid() {
    return m_rWindow.Exists() && m_rWindow.IsEnabled() && m_rWindow.IsVisible()
        && !m_rWindow.IsMinimized() && m_rWindow.IsForeground();
}

void CCursorHider::InnerShow() {
    if (m_bHide) {
        m_rWindow.ShowCursor();
        m_bHide = false;
    }
}

void CCursorHider::InnerHide() {
    if (!m_bHide) {
        m_rWindow.HideCursor();
        m_bHide = true;
    }
}

}
