#pragma once
namespace Themis {
namespace Detail {

//#define double float

struct SLanczosData {
    boost::scoped_array<int*> index;
    boost::scoped_array<int*> weight;
    boost::scoped_array<int> memory;
    int tap;
    void Set(int nTap, int nCount) {
        tap = nTap;
        index.reset(new int*[nCount]);
        weight.reset(new int*[nCount]);
        memory.reset(new int[nTap * nCount * 2]);
        int* p = memory.get();
        for (int i = 0; i < nCount; i++) {
            index[i] = p; p += nTap;
            weight[i] = p; p += nTap;
        }
    }
};

inline double LanczosWeight(double phase) {
    double a = fabs(phase);
    if (a < std::numeric_limits<double>::epsilon()) {
        return 1.0;
    }
    if (3.0 <= a) {
        return 0.0;
    }
    const double dPi = 3.141592653589793238;
    return sin(dPi * phase) * sin(dPi * phase / 3.0) / (dPi * dPi * phase * phase / 3.0);
}

inline void SetLanczosParamL(int nDstL, int nSrcL, SLanczosData& oData) {
    oData.Set(6, nDstL);
    const int nTap = oData.tap;
    boost::scoped_array<double> work(new double[nTap]);
    for (int i = 0; i < nDstL; i++) {
        double pos = ((i + 0.5) * nSrcL) / nDstL;
        int n = int(floor(pos - 2.5));
        pos = (n + 0.5 - pos);
        double sum = 0;
        for (int j = 0; j < nTap; j++) {
            if(n < 0) {
                oData.index[i][j] = 0;
            } else if(nSrcL <= n) {
                oData.index[i][j] = nSrcL - 1;
            } else {
                oData.index[i][j] = n;
            }
            work[j] = LanczosWeight(pos);
            sum += work[j];
            pos += 1;
            n += 1;
        }
        for (int j = 0; j < nTap; j++) {
            oData.weight[i][j] = int((work[j] / sum) * 0x10000);
        }
    }
}
inline void SetLanczosParamD(int nDstL, int nSrcL, SLanczosData& oData) {
    oData.Set((6 * nSrcL + (nDstL - 1)) / nDstL, nDstL);
    const int nTap = oData.tap;
    boost::scoped_array<double> work(new double[nTap]);
    for (int i = 0; i < nDstL; i++) {
        double pos = ((i - 3 + 0.5) * nSrcL) / nDstL + 0.5;
        int n = int(floor(pos));
        double sum = 0;
        for (int j = 0; j < nTap; j++) {
            double phase = ((n + 0.5) * nDstL) / nSrcL;
            phase -= (i + 0.5);
            if(n < 0) {
                oData.index[i][j] = 0;
            } else if(n >= nSrcL) {
                oData.index[i][j] = nSrcL - 1;
            } else {
                oData.index[i][j] = n;
            }
            work[j] = LanczosWeight(phase);
            sum += work[j];
            n += 1;
        }
        for (int j = 0; j < nTap; j++) {
            oData.weight[i][j] = int((work[j] / sum) * 0x10000);
        }
    }
}
inline void SetLanczosParam(int nDstL, int nSrcL, SLanczosData& oData) {
    if (nSrcL < nDstL) {
        SetLanczosParamL(nDstL, nSrcL, oData);
    } else {
        SetLanczosParamD(nDstL, nSrcL, oData);
    }
}

// ȉAweightintfloatɕςŁB

struct SLanczosData3DNow {
    boost::scoped_array<int*> index;
    boost::scoped_array<float*> weight;
    boost::scoped_array<int> memory;
    boost::scoped_array<float> memory2;
    int tap;
    void Set(int nTap, int nCount) {
        tap = nTap;
        index.reset(new int*[nCount]);
        weight.reset(new float*[nCount]);
        memory.reset(new int[nTap * nCount]);
        memory2.reset(new float[nTap * nCount]);
        int* p = memory.get();
        float* q = memory2.get();
        for (int i = 0; i < nCount; i++) {
            index[i] = p; p += nTap;
            weight[i] = q; q += nTap;
        }
    }
};

inline void SetLanczosParamL3DNow(int nDstL, int nSrcL, SLanczosData3DNow& oData) {
    oData.Set(6, nDstL);
    const int nTap = oData.tap;
    boost::scoped_array<double> work(new double[nTap]);
    for (int i = 0; i < nDstL; i++) {
        double pos = ((i + 0.5) * nSrcL) / nDstL;
        int n = int(floor(pos - 2.5));
        pos = (n + 0.5 - pos);
        double sum = 0;
        for (int j = 0; j < nTap; j++) {
            if(n < 0) {
                oData.index[i][j] = 0;
            } else if(nSrcL <= n) {
                oData.index[i][j] = nSrcL - 1;
            } else {
                oData.index[i][j] = n;
            }
            work[j] = LanczosWeight(pos);
            sum += work[j];
            pos += 1;
            n += 1;
        }
        for (int j = 0; j < nTap; j++) {
            oData.weight[i][j] = float(work[j] / sum);
        }
    }
}
inline void SetLanczosParamD3DNow(int nDstL, int nSrcL, SLanczosData3DNow& oData) {
    oData.Set((6 * nSrcL + (nDstL - 1)) / nDstL, nDstL);
    const int nTap = oData.tap;
    boost::scoped_array<double> work(new double[nTap]);
    for (int i = 0; i < nDstL; i++) {
        double pos = ((i - 3 + 0.5) * nSrcL) / nDstL + 0.5;
        int n = int(floor(pos));
        double sum = 0;
        for (int j = 0; j < nTap; j++) {
            double phase = ((n + 0.5) * nDstL) / nSrcL;
            phase -= (i + 0.5);
            if(n < 0) {
                oData.index[i][j] = 0;
            } else if(n >= nSrcL) {
                oData.index[i][j] = nSrcL - 1;
            } else {
                oData.index[i][j] = n;
            }
            work[j] = LanczosWeight(phase);
            sum += work[j];
            n += 1;
        }
        for (int j = 0; j < nTap; j++) {
            oData.weight[i][j] = float(work[j] / sum);
        }
    }
}
inline void SetLanczosParam3DNow(int nDstL, int nSrcL, SLanczosData3DNow& oData) {
    if (nSrcL < nDstL) {
        SetLanczosParamL3DNow(nDstL, nSrcL, oData);
    } else {
        SetLanczosParamD3DNow(nDstL, nSrcL, oData);
    }
}

inline void LanczosH(uint32_t* pDst, int nDstPitch, uint32_t* pSrc, int nSrcPitch,
int nDstW, int nSrcW, int startY, int nHeight, const SLanczosData& oData) {
    int tap = oData.tap;
    int** index = oData.index.get();
    int** weight = oData.weight.get();
    pDst += nDstPitch * startY;
    pSrc += nSrcPitch * startY;
    for (int y = startY; y < nHeight; y++) {
        for (int x = 0; x < nDstW; x++) {
            int dr = 0, dg = 0, db = 0;
            for (int i = 0; i < tap; i++) {
                const uint32_t dwSrc = pSrc[index[x][i]];
                Themis_PlaneGetBGR32(dwSrc, r, g, b);
                dr += r * weight[x][i];
                dg += g * weight[x][i];
                db += b * weight[x][i];
            }
            dr = (dr + 0x8000) >> 16;
            dg = (dg + 0x8000) >> 16;
            db = (db + 0x8000) >> 16;
            pDst[x] = Detail::BGR32(Detail::GetFixByte(dr), Detail::GetFixByte(dg), Detail::GetFixByte(db));
        }
        pDst += nDstPitch;
        pSrc += nSrcPitch;
    }
}

inline void LanczosHAlpha(uint32_t* pDst, int nDstPitch, uint32_t* pSrc, int nSrcPitch,
int nDstW, int nSrcW, int startY, int nHeight, const SLanczosData& oData) {
    int tap = oData.tap;
    int** index = oData.index.get();
    int** weight = oData.weight.get();
    pDst += nDstPitch * startY;
    pSrc += nSrcPitch * startY;
    for (int y = startY; y < nHeight; y++) {
        for (int x = 0; x < nDstW; x++) {
            int da = 0, dr = 0, dg = 0, db = 0;
            for (int i = 0; i < tap; i++) {
                const uint32_t dwSrc = pSrc[index[x][i]];
                Themis_PlaneGetBGRA32(dwSrc, a, r, g, b);
                da += a * weight[x][i];
                dr += r * weight[x][i];
                dg += g * weight[x][i];
                db += b * weight[x][i];
            }
            da = (da + 0x8000) >> 16;
            dr = (dr + 0x8000) >> 16;
            dg = (dg + 0x8000) >> 16;
            db = (db + 0x8000) >> 16;
            pDst[x] = Detail::BGRA32(Detail::GetFixByte(da), Detail::GetFixByte(dr), Detail::GetFixByte(dg), Detail::GetFixByte(db));
        }
        pDst += nDstPitch;
        pSrc += nSrcPitch;
    }
}

inline void LanczosV(uint32_t* pDst, int nDstPitch, uint32_t* pSrc, int nSrcPitch,
int nWidth, int startY, int endDstY, int nSrcH, const SLanczosData& oData) {
    int tap = oData.tap;
    int** index = oData.index.get();
    int** weight = oData.weight.get();
    pDst += nDstPitch * startY;
    for (int y = startY; y < endDstY; y++) {
        for (int x = 0; x < nWidth; x++) {
            int dr = 0, dg = 0, db = 0;
            for (int i = 0; i < tap; i++) {
                const uint32_t dwSrc = pSrc[x + nSrcPitch * index[y][i]];
                Themis_PlaneGetBGR32(dwSrc, r, g, b);
                dr += r * weight[y][i];
                dg += g * weight[y][i];
                db += b * weight[y][i];
            }
            dr = (dr + 0x8000) >> 16;
            dg = (dg + 0x8000) >> 16;
            db = (db + 0x8000) >> 16;
            pDst[x] = Detail::BGR32(Detail::GetFixByte(dr), Detail::GetFixByte(dg), Detail::GetFixByte(db));
        }
        pDst += nDstPitch;
    }
}

inline void LanczosVAlpha(uint32_t* pDst, int nDstPitch, uint32_t* pSrc, int nSrcPitch,
int nWidth, int startY, int endDstY, int nSrcH, const SLanczosData& oData) {
    int tap = oData.tap;
    int** index = oData.index.get();
    int** weight = oData.weight.get();
    pDst += nDstPitch * startY;
    for (int y = startY; y < endDstY; y++) {
        for (int x = 0; x < nWidth; x++) {
            int da = 0, dr = 0, dg = 0, db = 0;
            for (int i = 0; i < tap; i++) {
                const uint32_t dwSrc = pSrc[x + nSrcPitch * index[y][i]];
                Themis_PlaneGetBGRA32(dwSrc, a, r, g, b);
                da += a * weight[y][i];
                dr += r * weight[y][i];
                dg += g * weight[y][i];
                db += b * weight[y][i];
            }
            da = (da + 0x8000) >> 16;
            dr = (dr + 0x8000) >> 16;
            dg = (dg + 0x8000) >> 16;
            db = (db + 0x8000) >> 16;
            pDst[x] = Detail::BGRA32(Detail::GetFixByte(da), Detail::GetFixByte(dr), Detail::GetFixByte(dg), Detail::GetFixByte(db));
        }
        pDst += nDstPitch;
    }
}

#undef double

}//namespace Detail
}
