#pragma once
#include "../../Windows/CpuId.h"
#include "Lanczos3.h"
namespace Themis {
namespace Detail {

inline void MMXAlphaFill32(uint32_t* pDst, int nDstPitch, int nMMXSize, int nDstH, uint32_t dwColor, int nAlpha) {
    const uint32_t dwAlpha = (nAlpha << 16) | nAlpha;
    const uint32_t qwAlpha[] = {dwAlpha, dwAlpha};
    const uint32_t qwColor[] = {dwColor, dwColor};
    for (int y = 0; y < nDstH; y++) {
        _asm {
            mov     edi, pDst
            mov     eax, nMMXSize
            xor     ecx, ecx
            pxor    mm0, mm0
            movq    mm7, qwAlpha
        ___loop:
                movq        mm1, qwColor
                movq        mm2, qwColor
                movq        mm3, [edi + ecx * 8]
                movq        mm4, [edi + ecx * 8]
                punpcklbw   mm1, mm0
                punpckhbw   mm2, mm0
                punpcklbw   mm3, mm0
                punpckhbw   mm4, mm0
                psubw       mm1, mm3
                psubw       mm2, mm4
                pmullw      mm1, mm7
                pmullw      mm2, mm7
                psrlw       mm1, 8
                psrlw       mm2, 8
                paddb       mm1, mm3
                paddb       mm2, mm4
                packuswb    mm1, mm2
                movq        [edi + ecx * 8], mm1
            inc     ecx
            cmp     eax, ecx
            jnz     ___loop
        }
        pDst += nDstPitch;
    }
    _asm emms
}

inline void MMXSatAddFill32(uint32_t* pDst, int nDstPitch, int nMMXSize, int nDstH, uint32_t dwColor) {
    const uint32_t qwColor[] = {dwColor, dwColor};
    for (int y = 0; y < nDstH; y++) {
        _asm {
            mov     edi, pDst
            mov     eax, nMMXSize
            xor     ecx, ecx
            pxor    mm0, mm0
            movq    mm2, qwColor
        ___loop:
                movq        mm1, [edi + ecx * 8]
                paddusb     mm1, mm2
                movq        [edi + ecx * 8], mm1
            inc     ecx
            cmp     eax, ecx
            jnz     ___loop
        }
        pDst += nDstPitch;
    }
    _asm emms
}

inline void MMXSatSubFill32(uint32_t* pDst, int nDstPitch, int nMMXSize, int nDstH, uint32_t dwColor) {
    const uint32_t qwColor[] = {dwColor, dwColor};
    for (int y = 0; y < nDstH; y++) {
        _asm {
            mov     edi, pDst
            mov     eax, nMMXSize
            xor     ecx, ecx
            pxor    mm0, mm0
            movq    mm2, qwColor
        ___loop:
                movq        mm1, [edi + ecx * 8]
                psubusb     mm1, mm2
                movq        [edi + ecx * 8], mm1
            inc     ecx
            cmp     eax, ecx
            jnz     ___loop
        }
        pDst += nDstPitch;
    }
    _asm emms
}

inline void MMXSatAddBlt32(uint32_t* pDst, int nDstPitch, uint32_t* pSrc, int nSrcPitch,
int nMMXSize, int nDstH) {
    for (int y = 0; y < nDstH; y++) {
        _asm {
            mov     edi, pDst
            mov     esi, pSrc
            mov     eax, nMMXSize
            xor     ecx, ecx
        ___loop:
                movq        mm1, [edi + ecx * 8]
                movq        mm2, [esi + ecx * 8]
                paddusb     mm1, mm2
                movq        [edi + ecx * 8], mm1
            inc     ecx
            cmp     eax, ecx
            jnz     ___loop
        }
        pDst += nDstPitch;
        pSrc += nSrcPitch;
    }
    _asm emms
}

inline void MMXSatSubBlt32(uint32_t* pDst, int nDstPitch, uint32_t* pSrc, int nSrcPitch,
int nMMXSize, int nDstH) {
    for (int y = 0; y < nDstH; y++) {
        _asm {
            mov     edi, pDst
            mov     esi, pSrc
            mov     eax, nMMXSize
            xor     ecx, ecx
        ___loop:
                movq        mm1, [edi + ecx * 8]
                movq        mm2, [esi + ecx * 8]
                psubusb     mm1, mm2
                movq        [edi + ecx * 8], mm1
            inc     ecx
            cmp     eax, ecx
            jnz     ___loop
        }
        pDst += nDstPitch;
        pSrc += nSrcPitch;
    }
    _asm emms
}

inline void MMXFadeBlt32(uint32_t* pDst, int nDstPitch, uint32_t* pSrc, int nSrcPitch,
int nMMXSize, int nDstH, int nFade) {
    const uint32_t dwFade = (nFade<<16) | nFade;
    const uint32_t qwFade[] = {dwFade, dwFade};
    for (int y = 0; y < nDstH; y++) {
        _asm {
            mov     edi, pDst
            mov     esi, pSrc
            mov     eax, nMMXSize
            xor     ecx, ecx
            movq    mm0, qwFade
            pxor    mm7, mm7
        ___loop:
                movq        mm1, [esi + ecx * 8]
                movq        mm2, [esi + ecx * 8]
                punpcklbw   mm1, mm7
                punpckhbw   mm2, mm7
                pmullw      mm1, mm0
                pmullw      mm2, mm0
                psrlw       mm1, 8
                psrlw       mm2, 8
                packuswb    mm1, mm2
                movq        [edi + ecx * 8], mm1
            inc     ecx
            cmp     eax, ecx
            jnz     ___loop
        }
        pDst += nDstPitch;
        pSrc += nSrcPitch;
    }
    _asm emms
}

inline void MMXBrightBlt32(uint32_t* pDst, int nDstPitch, uint32_t* pSrc, int nSrcPitch,
int nMMXSize, int nDstH, int nBright) {
    const uint32_t dwBright = (nBright<<16) | nBright;
    const uint32_t qwBright[] = {dwBright, dwBright};
    const uint32_t qwPckMask[] = {0x00ff00ff, 0x00ff00ff};
    for (int y = 0; y < nDstH; y++) {
        _asm {
            mov     edi, pDst
            mov     esi, pSrc
            mov     eax, nMMXSize
            xor     ecx, ecx
            pxor    mm0, mm0
            movq    mm7, qwBright
        ___loop:
                movq        mm1, qwPckMask
                movq        mm2, qwPckMask
                movq        mm3, [esi + ecx * 8]
                movq        mm4, [esi + ecx * 8]
                punpcklbw   mm3, mm0
                punpckhbw   mm4, mm0
                psubw       mm1, mm3
                psubw       mm2, mm4
                pmullw      mm1, mm7
                pmullw      mm2, mm7
                psrlw       mm1, 8
                psrlw       mm2, 8
                paddb       mm1, mm3
                paddb       mm2, mm4
                packuswb    mm1, mm2
                movq        [edi + ecx * 8], mm1
            inc     ecx
            cmp     eax, ecx
            jnz     ___loop
        }
        pDst += nDstPitch;
        pSrc += nSrcPitch;
    }
    _asm emms
}

inline void MMXAlphaBlt32(uint32_t* pDst, int nDstPitch, uint32_t* pSrc1, int nSrcPitch1,
uint32_t* pSrc2, int nSrcPitch2, int nMMXSize, int nDstH, int nAlpha) {
    const uint16_t qwAlpha[4] = {nAlpha, nAlpha, nAlpha, nAlpha};
    for (int y = 0; y < nDstH; y++) {
        _asm {
            mov     edx, pDst
            mov     edi, pSrc1
            mov     esi, pSrc2
            mov     eax, nMMXSize
            xor     ecx, ecx
            pxor    mm0, mm0
            movq    mm7, qwAlpha
        ___loop:
                movq        mm1, [esi + ecx * 8]
                movq        mm3, [esi + ecx * 8]
                movq        mm2, [edi + ecx * 8]
                movq        mm4, [edi + ecx * 8]
                punpcklbw   mm1, mm0
                punpckhbw   mm3, mm0
                punpcklbw   mm2, mm0
                punpckhbw   mm4, mm0
                psubw       mm1, mm2
                psubw       mm3, mm4
                pmullw      mm1, mm7
                pmullw      mm3, mm7
                psrlw       mm1, 8
                psrlw       mm3, 8
                paddb       mm1, mm2
                paddb       mm3, mm4
                packuswb    mm1, mm3
                movq        [edx + ecx * 8], mm1
            inc     ecx
            cmp     eax, ecx
            jnz     ___loop
        }
        pDst += nDstPitch;
        pSrc1 += nSrcPitch1;
        pSrc2 += nSrcPitch2;
    }
    _asm emms
}

inline void MMXClearTypeBlt32(uint32_t* pDst, int nDstPitch, uint32_t* pSrc, int nSrcPitch,
int nMMXSize, int nDstH) {
    const uint32_t qwRMask[2] = {0x00ff0000, 0x00ff0000};
    const uint32_t qwGMask[2] = {0xff00ff00, 0xff00ff00}; // łɃlꂿႤ
    const uint32_t qwBMask[2] = {0x000000ff, 0x000000ff};
    if (CCpuId::IsE3DNow()) {
        _asm femms
        for (int y = 0; y < nDstH; y++) {
            _asm {
                mov     edi, pDst
                mov     esi, pSrc
                mov     eax, nMMXSize
                xor     ecx, ecx
                xor     edx, edx
                movq    mm5, qwRMask
                movq    mm6, qwGMask
                movq    mm7, qwBMask
            ___loop2:
                    movq    mm0, [esi + edx]        ; 1, 2
                    pswapd  mm1, [esi + edx + 8]    ; 4, 3
                    movq    mm2, [esi + edx + 16]   ; 5, 6
                    pswapd  mm3, mm0                ; 2, -(1)
                    punpckldq   mm0, mm1            ; 1, 4
                    punpckhdq   mm1, mm2            ; 3, 6
                    punpckldq   mm3, mm2            ; 2, 5
                    pand    mm3, mm6                ; 2, 5 & g
                    pand    mm0, mm5                ; 1, 4 & r
                    pand    mm1, mm7                ; 3, 6 & b
                    por     mm0, mm1
                    por     mm3, mm0
                    movntq  [edi + ecx * 8], mm3    ; ݃I[̏ꍇɃLbVŏɂmovq (MMX2)
                inc     ecx
                add     edx, 24
                cmp     eax, ecx
                jnz     ___loop2
            }
            pDst += nDstPitch;
            pSrc += nSrcPitch;
        }
        _asm femms
    } else {
        for (int y = 0; y < nDstH; y++) {
            _asm {
                mov     edi, pDst
                mov     esi, pSrc
                mov     eax, nMMXSize
                xor     ecx, ecx
                xor     edx, edx
                movq    mm5, qwRMask
                movq    mm6, qwGMask
                movq    mm7, qwBMask
            ___loop:
                ; rotate΂傢ôǁBB
                    movq    mm0, [esi + edx]        ; 1, 2
                    movq    mm1, [esi + edx + 8]    ; 3, 4
                    movq    mm2, [esi + edx + 16]   ; 5, 6
                    movq    mm3, mm0                ; 1, 2
                    punpckldq mm4, mm1
                    punpckhdq mm1, mm4              ; 4, 3
                    psrlq   mm0, 32                 ; 2, -
                    punpckldq   mm3, mm1            ; 1, 4
                    punpckldq   mm0, mm2            ; 2, 5
                    punpckhdq   mm1, mm2            ; 3, 6
                    pand    mm0, mm6                ; 2, 5 & g
                    pand    mm3, mm5                ; 1, 4 & r
                    pand    mm1, mm7                ; 3, 6 & b
                    por     mm3, mm1
                    por     mm0, mm3
                    movq    [edi + ecx * 8], mm0
                inc     ecx
                add     edx, 24
                cmp     eax, ecx
                jnz     ___loop
            }
            pDst += nDstPitch;
            pSrc += nSrcPitch;
        }
        _asm emms
    }
}

#if 1
# define ___h_prefetchw prefetchw
#else
# define ___h_prefetchw ;
#endif
#if 0
# define ___h_prefetch prefetch
#else
# define ___h_prefetch ;
#endif
#if 1
# define ___v_prefetchw prefetchw
#else
# define ___v_prefetchw ;
#endif
#if 1
# define ___v_prefetch prefetch
#else
# define ___v_prefetch ;
#endif

inline void _3DNowLanczosH(uint32_t* pDst, int nDstPitch, uint32_t* pSrc, int nSrcPitch,
int nDstW, int nSrcW, int startY, int endY, const SLanczosData3DNow& oData) {
    int tap = oData.tap;
    int** index = oData.index.get();
    float** weight = oData.weight.get();
    pDst += nDstPitch * startY;
    pSrc += nSrcPitch * startY;
    const uint32_t qwGBMask[2] = {0x000000ff, 0x000000ff};
    const uint32_t qwARMask[2] = {0x00ff0000, 0x00ff0000};
    const float qw0_5f[2]   = {0.5f, 0.5f};
    _asm femms
    for (int y = startY; y < endY; y++) {
        _asm {
            xor     ecx, ecx                    ; ecx := x
            mov     esi, pSrc
            ___h_prefetchw pDst
        ___loop:
                pxor    mm0, mm0                ; (00000000, 00000000)
                mov     eax, index
                mov     edx, weight
                mov     eax, [eax + ecx * 4]    ; index[x]
                mov     edx, [edx + ecx * 4]    ; weight[x]
                xor     ebx, ebx                ; i
                ___h_prefetch byte ptr [eax]
                ___h_prefetch byte ptr [edx]
                pi2fd   mm6, mm0                ; float(dg = 0.0, db = 0.0)
                pi2fd   mm7, mm0                ; float(da = 0.0, dr = 0.0)
            ___loop2:
                    mov         edi, [eax + ebx * 4]    ; index[x][i]
                    movd        mm0, [esi + edi * 4]    ; pSrc[index[x][i]]
                    movd        mm1, [esi + edi * 4]    ; pSrc[index[x][i]]
                    movq        mm2, mm0
                    movq        mm3, mm1
                    psrlq       mm2, 8
                    psrlq       mm3, 8
                    punpckldq   mm0, mm2
                    punpckldq   mm1, mm3
                    pand        mm0, qwGBMask           ; (000000gg, 000000bb)
                    pand        mm1, qwARMask           ; (00aa0000, 00rr0000)
                    movd        mm2, [edx + ebx * 4]    ; weight[x][i]
                    movd        mm3, [edx + ebx * 4]    ; weight[x][i]
                    psrld       mm1, 16                 ; (000000aa, 000000rr)
                    punpckldq   mm2, mm3                ; float(weight[x][i], weight[x][i])
                    pi2fd       mm4, mm0                ; float(000000gg, 000000bb)
                    pi2fd       mm5, mm1                ; float(000000aa, 000000rr)
                    pfmul       mm4, mm2                ; float(000000gg, 000000bb) * weight
                    pfmul       mm5, mm2                ; float(000000aa, 000000rr) * weight
                    pfadd       mm6, mm4                ; float(dg += gg, db += bb)
                    pfadd       mm7, mm5                ; float(da += aa, dr += rr)
                inc     ebx
                cmp     ebx, tap
                jnz     ___loop2
                mov     edi, pDst
                paddd   mm0, qw0_5f
                paddd   mm1, qw0_5f
                pf2id   mm0, mm6            ; int(dg, db)    pf2id ̓[ɌĊۂ߂
                pf2id   mm1, mm7            ; int(da, dr)
                packssdw mm0, mm1
                pxor    mm3, mm3
                packuswb mm0, mm3
                movd    [edi + ecx * 4], mm0    ; pDst[x] = mm
            inc     ecx
            cmp     nDstW, ecx
            jnz     ___loop
        }
        pDst += nDstPitch;
        pSrc += nSrcPitch;
    }
    _asm femms
}

inline void _3DNowLanczosV(uint32_t* pDst, int nDstPitch, uint32_t* pSrc, int nSrcPitch,
int nWidth, int startY, int endDstY, int nSrcH, const SLanczosData3DNow& oData) {
    int tap = oData.tap;
    int** index = oData.index.get();
    float** weight = oData.weight.get();
    pDst += nDstPitch * startY;
    const uint32_t qwGBMask[2] = {0x000000ff, 0x000000ff};
    const uint32_t qwARMask[2] = {0x00ff0000, 0x00ff0000};
    const float qw0_5f[2]   = {0.5f, 0.5f};
    _asm femms
    for (int y = startY; y < endDstY; y++) {
        _asm {
            xor     ecx, ecx                    ; ecx := x
            mov     esi, pSrc
            mov     eax, index
            mov     edx, weight
            mov     ebx, y
            mov     eax, [eax + ebx * 4]    ; index[y]
            mov     edx, [edx + ebx * 4]    ; weight[y]
            ___v_prefetchw pDst
            ___v_prefetch byte ptr [eax]
            ___v_prefetch byte ptr [edx]
        ___loop:
                xor     ebx, ebx                ; i
                pxor    mm0, mm0                ; (00000000, 00000000)
                pi2fd   mm6, mm0                ; float(dg = 0.0, db = 0.0)
                pi2fd   mm7, mm0                ; float(da = 0.0, dr = 0.0)
            ___loop2:
                    mov         edi, nSrcPitch
                    push        eax
                    push        edx
                        mov         eax, [eax + ebx * 4]    ; index[y][i]
                        mul         edi                     ; nSrcPitch * index[y][i]
                        lea         edi, [ecx + eax]        ; x + nSrcPitch * index[y][i]
                    pop         edx
                    pop         eax
                    movd        mm0, [esi + edi * 4]    ; pSrc[x + nSrcPitch * index[y][i]]
                    movd        mm1, [esi + edi * 4]    ; pSrc[x + nSrcPitch * index[y][i]]
                    movq        mm2, mm0
                    movq        mm3, mm1
                    psrlq       mm2, 8
                    psrlq       mm3, 8
                    punpckldq   mm0, mm2
                    punpckldq   mm1, mm3
                    pand        mm0, qwGBMask           ; (000000gg, 000000bb)
                    pand        mm1, qwARMask           ; (00aa0000, 00rr0000)
                    movd        mm2, [edx + ebx * 4]    ; weight[y][i]
                    movd        mm3, [edx + ebx * 4]    ; weight[y][i]
                    psrld       mm1, 16                 ; (000000aa, 000000rr)
                    punpckldq   mm2, mm3                ; float(weight[y][i], weight[y][i])
                    pi2fd       mm4, mm0                ; float(000000gg, 000000bb)
                    pi2fd       mm5, mm1                ; float(000000aa, 000000rr)
                    pfmul       mm4, mm2                ; float(000000gg, 000000bb) * weight
                    pfmul       mm5, mm2                ; float(000000aa, 000000rr) * weight
                    pfadd       mm6, mm4                ; float(dg += gg, db += bb)
                    pfadd       mm7, mm5                ; float(da += aa, dr += rr)
                inc     ebx
                cmp     ebx, tap
                jnz     ___loop2
                mov     edi, pDst
                paddd   mm0, qw0_5f
                paddd   mm1, qw0_5f
                pf2id   mm0, mm6            ; int(dg, db)    pf2id ̓[ɌĊۂ߂
                pf2id   mm1, mm7            ; int(da, dr)
                packssdw mm0, mm1
                pxor    mm3, mm3
                packuswb mm0, mm3
                movd    [edi + ecx * 4], mm0    ; pDst[x] = mm
            inc     ecx
            cmp     ecx, nWidth
            jnz     ___loop
        }
        pDst += nDstPitch;
    }
    _asm femms
}

}//namespace Detail
}
