#include "stdafx.h"
#include "../Graphics.h"
#include "../Image.h"
namespace Themis {

CImageLoader::CImageLoader(): m_nRenderFlag(0) {
}

CImageLoader::~CImageLoader() {
}

void CImageLoader::SetRenderOnlyAtOnce(bool b) {
    Themis_DebugAssert(m_nRenderFlag != 2);
    m_nRenderFlag = b ? 1 : 0;
}

bool CImageLoader::IsRenderOnlyAtOnce() const {
    return m_nRenderFlag != 0;
}

void CImageLoader::Load(const std::tr1::shared_ptr<CFileData>& pFileData) {
    if (m_nRenderFlag == 2) { // `ς݂Ȃ疢`
        m_nRenderFlag = 1;
    }

    m_pFileData = pFileData;
    // Xgɓo^Ă̂Ɏs
    if (0 < m_pFileData->GetMemory().size()) {
        FactoryList<IImageLoader>& rFactoryList = GetFactoryList();
        for (int i = 0, n = rFactoryList.GetCount(); i < n; i++) {
            try {
                m_pLoader = rFactoryList.CreateInstance(i);
                if (m_pLoader->Load(m_pFileData, IsRenderOnlyAtOnce())) {
                    return;
                }
            } catch (std::exception& e) {
                Themis_DebugLog(e.what()); e;
                // G[oۂǖĎ
            }
        }
    } else {
        Themis_DebugLog(L"摜̃t@CTCYÔœǂݍݎs");
    }
    // ǂݍ߂̖
    m_pLoader.reset();
    throw ImageLoadException(m_pFileData->GetName());
}

CSize CImageLoader::GetSize() const {
    if (m_pLoader == NULL) {
        throw ImageLoadException();
    }
    return m_pLoader->GetSize();
}

void CImageLoader::Render(CPlane& dst, const CPoint& pt) {
    if (m_pLoader == NULL) {
        throw ImageLoadException();
    }
    switch (m_nRenderFlag) {
    case 0:
        m_pLoader->Render(dst, pt, false);
        break;
    case 1:
        m_nRenderFlag = 2;
        m_pLoader->Render(dst, pt, true);
        break;
    case 2:
        throw ImageLoadException();
     // throw RuntimeException(L"摜̍ĕ`Ɏs܂");
    default:
        Themis_DebugTrespass();
        break;
    }
}

bool CImageLoader::HasAlpha() const {
    if (m_pLoader == NULL) {
        throw ImageLoadException();
    }
    return m_pLoader->HasAlpha();
}

string CImageLoader::GetTypeName() const {
    if (m_pLoader == NULL) {
        throw ImageLoadException();
    }
    return m_pLoader->GetTypeName();
}

CSize CImageLoader::Load(CPlane& rPlane, const std::tr1::shared_ptr<CFileData>& pFileData) {
    CImageLoader oLoader;
    oLoader.Load(pFileData);
    CSize size = oLoader.GetSize();
    rPlane.SetSize(size);
    oLoader.Render(rPlane);
    return size;
}

/// ΉĂƎv킵gq̃Xg擾
vector<string> CImageLoader::GetSupportedSuffixesLc() {
    vector<string> result;
    FactoryList<IImageLoader>& rFactoryList = GetFactoryList();
    for (int i = 0, n = rFactoryList.GetCount(); i < n; i++) {
        try {
            std::tr1::shared_ptr<IImageLoader> loader = rFactoryList.CreateInstance(i);
            CSpi::SplitSuffixes(result, loader->GetTypeFileName());
        } catch (std::exception& e) {
            Themis_DebugLog(e.what()); e;
            // G[oۂǖĎ
        }
    }
    std::sort(result.begin(), result.end());
    result.erase(std::unique(result.begin(), result.end()), result.end());
    return result;
}

FactoryList<IImageLoader>& CImageLoader::GetFactoryList() {
    static FactoryList<IImageLoader> oList;
    return oList;
}

}
