#include "stdafx.h"
#include "Jpg.h"
#include "../File.h"
#include "../Graphics.h"
#include "../String/Encoding.h"
#if Themis_ConfigJpgLib
# include "../../../external/jpeglib.h"
# ifdef NDEBUG
#  pragma comment(lib, "jpeg_simd_mr.lib")
# else
#  pragma comment(lib, "jpeg_simd_md.lib")
# endif
#endif
namespace Themis {

#if Themis_ConfigJpgLib

namespace {
    METHODDEF(void) my_error_exit(j_common_ptr cinfo) {
        throw ImageLoadException();
        //jpeglib̃\[X̒˂؂ĔłႤǁA
        //longjmpłsۂv낤AƁB
    }
    METHODDEF(void) my_output_message(j_common_ptr cinfo) {
      #ifndef NDEBUG
        char buffer[JMSG_LENGTH_MAX];
        (*cinfo->err->format_message)(cinfo, buffer);
        Themis_DebugLog(Encoding::GetDefault().GetString(buffer));
      #endif
    }
}
CJpgLoader::CJpgLoader(): m_pcinfo(new jpeg_decompress_struct), m_pjerr(new jpeg_error_mgr) {
    m_pcinfo->err = jpeg_std_error(m_pjerr.get());
    m_pjerr->error_exit = my_error_exit;
 // m_pjerr->output_message = my_output_message;
    jpeg_create_decompress(m_pcinfo.get());
}

CJpgLoader::~CJpgLoader() {
    jpeg_destroy_decompress(m_pcinfo.get());
}

//jdatasrc.cQlɁAKɁBB
namespace {
    typedef struct {
        struct jpeg_source_mgr pub;   //public fields
        CFileData* fileData;
        bool bFirst;
        JOCTET endbuf[2];
    } my_source_mgr, *my_source_mgr_ptr;

    METHODDEF(void) my_init_source(j_decompress_ptr cinfo) {
        my_source_mgr_ptr src = (my_source_mgr_ptr) cinfo->src;
        src->bFirst = true;
        src->endbuf[0] = 0xFF;
        src->endbuf[1] = JPEG_EOI;
    }
    METHODDEF(boolean) my_fill_input_buffer(j_decompress_ptr cinfo) {
        my_source_mgr_ptr src = (my_source_mgr_ptr) cinfo->src;
        if (src->bFirst) {
            src->bFirst = false;
            const std::vector<uint8_t>& vctMemory = src->fileData->GetMemory();
            src->pub.next_input_byte = &vctMemory[0];
            src->pub.bytes_in_buffer = vctMemory.size();
        } else {
            src->pub.next_input_byte = src->endbuf;
            src->pub.bytes_in_buffer = 2;
        }
        return TRUE;
    }
    METHODDEF(void) my_skip_input_data(j_decompress_ptr cinfo, long num_bytes) {
        my_source_mgr_ptr src = (my_source_mgr_ptr) cinfo->src;
        if (num_bytes > 0) {
            while (num_bytes > (long) src->pub.bytes_in_buffer) {
                num_bytes -= (long) src->pub.bytes_in_buffer;
                (void) my_fill_input_buffer(cinfo);
            }
            src->pub.next_input_byte += (size_t) num_bytes;
            src->pub.bytes_in_buffer -= (size_t) num_bytes;
        }
    }
    METHODDEF(void) my_term_source(j_decompress_ptr cinfo) {
        my_source_mgr_ptr src = (my_source_mgr_ptr) cinfo->src;
    }
    void jpeg_my_src(jpeg_decompress_struct* cinfo, CFileData* fileData) {
        my_source_mgr_ptr src;
        if (cinfo->src == NULL) {   /* first time for this JPEG object? */
            cinfo->src = (struct jpeg_source_mgr *)(*cinfo->mem->alloc_small) ((j_common_ptr) cinfo, JPOOL_PERMANENT, sizeof(my_source_mgr));
            src = (my_source_mgr_ptr) cinfo->src;
        }
        src = (my_source_mgr_ptr)cinfo->src;
        src->pub.init_source = my_init_source;
        src->pub.fill_input_buffer = my_fill_input_buffer;
        src->pub.skip_input_data = my_skip_input_data;
        src->pub.resync_to_restart = jpeg_resync_to_restart; // use default method
        src->pub.term_source = my_term_source;
        src->fileData = fileData;
        src->pub.bytes_in_buffer = 0; // forces fill_input_buffer on first read
        src->pub.next_input_byte = NULL; // until buffer loaded
    }
}

bool CJpgLoader::Load(const std::tr1::shared_ptr<CFileData>& fileData, bool bRenderOnlyAtOnce) {
    try {
    /*
        FILE* fp = rFile.GetHandle();
        fseek(fp, 0, SEEK_SET);
        jpeg_stdio_src(m_pcinfo.get(), fp);         // ̓t@C̐ݒ
    /*/
        this->fileData = fileData;
        jpeg_my_src(m_pcinfo.get(), fileData.get());      // ̓t@C̐ݒ
    //*/
        jpeg_read_header(m_pcinfo.get(), TRUE);     // t@C̏wb_̓Ǎ
        if (!bRenderOnlyAtOnce) {
            m_pPlane.reset(new CMemoryPlane(32));
            m_pPlane->SetSize(GetSize());
            Decompress(*m_pPlane, CPoint());
        }
        return true;
    } catch (std::exception&) {
        return false;
    }
}

CSize CJpgLoader::GetSize() const {
    return CSize(m_pcinfo->image_width, m_pcinfo->image_height);
}

void CJpgLoader::Render(CPlane& dst, const CPoint& pt, bool bRenderOnlyAtOnce) {
    if (bRenderOnlyAtOnce) {
        try {
            if (dst.GetBpp() != 32) {
                CMemoryPlane oTmpPlane(32);
                CSize img = GetSize();
                CSize pln = dst.GetSize() - pt;
                oTmpPlane.SetSize(CSize(Min(img.cx, pln.cx), Min(img.cy, pln.cy)));  //ɍ킹
                Decompress(oTmpPlane, CPoint());
                dst.Blt(oTmpPlane, pt, CRect(oTmpPlane.GetSize()));
            } else {
                Decompress(dst, pt);
            }
        } catch (std::exception&) {
            throw ImageLoadException();
        }
    } else {
        dst.Blt(*m_pPlane, pt, m_pPlane->GetSize().ToRect());
    }
}

void CJpgLoader::Decompress(CPlane& dst, const CPoint& pt) const {
    jpeg_start_decompress(m_pcinfo.get());      // 𓀂̊Jn

    int row_stride = m_pcinfo->output_width * m_pcinfo->output_components;              // o̓obt@̕s
    JSAMPLE** buffer = (*m_pcinfo->mem->alloc_sarray)(
        reinterpret_cast<j_common_ptr>(m_pcinfo.get()), JPOOL_IMAGE, row_stride, 1);    // so̓obt@̍쐬
    if (buffer == NULL) {
        throw ImageLoadException();
    }
    Themis_PlaneLock4(dst, pt, p, nPitch);
    // PsǂŃv[ցB
    if (m_pcinfo->output_components == 3) {         //24bpp
        while (m_pcinfo->output_scanline < m_pcinfo->output_height) {
            jpeg_read_scanlines(m_pcinfo.get(), buffer, 1);
            uint8_t* pSrc = buffer[0];
            for(int x = 0, x3 = 0, cx = m_pcinfo->image_width; x < cx; x++, x3 += 3) {
                p[x] = (pSrc[x3] << 16) | (pSrc[x3+1] << 8) | pSrc[x3 + 2];
            }
            p += nPitch;
        }
    } else if (m_pcinfo->output_components == 1) {    //8bpp
        while (m_pcinfo->output_scanline < m_pcinfo->output_height) {
            jpeg_read_scanlines(m_pcinfo.get(), buffer, 1);
            uint8_t* pSrc = buffer[0];
            for(int x = 0, cx = m_pcinfo->image_width; x < cx; x++) {
                p[x] = (pSrc[x]<<16) | (pSrc[x]<<8) | pSrc[x];   //| 0xff000000
            }
            p += nPitch;
        }
    } else {
        jpeg_finish_decompress(m_pcinfo.get());
        Themis_DebugWarning(L"JPEGt@CT|[gȌ`ł");
        throw ImageLoadException();
    }

    jpeg_finish_decompress(m_pcinfo.get());
}


CJpgSaver::CJpgSaver(): m_pcinfo(new jpeg_compress_struct), m_pjerr(new jpeg_error_mgr) {
    m_pcinfo->err = jpeg_std_error(m_pjerr.get());
    m_pjerr->error_exit = my_error_exit;
    jpeg_create_compress(m_pcinfo.get());
    m_pcinfo->input_components = 3;         // sNZ̐F
    m_pcinfo->in_color_space = JCS_RGB;     // J[Xy[X
    jpeg_set_defaults(m_pcinfo.get());      // ftHgkp[^ݒ
}

CJpgSaver::~CJpgSaver() {
    jpeg_destroy_compress(m_pcinfo.get());
}

CJpgSaver& CJpgSaver::SetQuality(int nQuality) {
    jpeg_set_quality(m_pcinfo.get(), nQuality, TRUE);
    return *this;
}

CJpgSaver& CJpgSaver::SetProgress() {
    jpeg_simple_progression(m_pcinfo.get());
    return *this;
}

void CJpgSaver::Save(const CFileName& oFileName, const CPlane& src, const CRect& rc) {
    try {
        // 32ȊOȂv[ē][
        std::tr1::shared_ptr<const CPlane> pPlane;
        if (src.GetBpp() != 32) {
            CMemoryPlane oTmpPlane(32);
            oTmpPlane.SetSize(rc.GetSize());
            oTmpPlane.Blt(src, CPoint(), rc);
            Save(oFileName, oTmpPlane, rc.GetSize().ToRect());
        } else {
            // t@C|C^E摜̃TCYZbg
            CFile(oFileName).CreateDir();
            CStdFile oFile(oFileName, L"wb");
            FILE* fp = oFile.GetHandle();
            jpeg_stdio_dest(m_pcinfo.get(), fp);
            m_pcinfo->image_width = rc.GetWidth();
            m_pcinfo->image_height = rc.GetHeight();

            // kJn
            jpeg_start_compress(m_pcinfo.get(), TRUE);
            boost::scoped_array<JSAMPLE> abyBuffer(new JSAMPLE[rc.GetWidth() * 3]);

            Themis_PlaneLock4(src, rc.GetPoint(), p, nPitch);
            Themis_PlaneGetSize(rc.GetSize(), nSrcW, nSrcH);
            for (int y = 0; y < nSrcH; y++) {
                for(int x = 0, x3 = 0; x < nSrcW; x++, x3 += 3) {
                    abyBuffer[x3 + 0] = JSAMPLE((p[x] >> 16) & 0xff);
                    abyBuffer[x3 + 1] = JSAMPLE((p[x] >> 8) & 0xff);
                    abyBuffer[x3 + 2] = JSAMPLE(p[x] & 0xff);
                }
                p += nPitch;
                JSAMPROW row_pointer = abyBuffer.get(); // pointer to JSAMPLE row[s]
                jpeg_write_scanlines(m_pcinfo.get(), &row_pointer, 1);
            }

            // kI
            jpeg_finish_compress(m_pcinfo.get());
        }
    } catch (std::exception&) {
        throw ImageSaveException(oFileName);
    }
}

#endif//Themis_ConfigJpgLib

}
