#include "stdafx.h"
#include "KeyTable.h"
#include "String/StringBuilder.h"
#include "Utility/StringUtility.h"
namespace Themis {

struct SKeyTableData {
    std::map<UINT, CKeyTable::EKey> oVKeyToEKey;
    TwowayMap<CKeyTable::EKey, string> oKeyNameList;
    SKeyTableData() {
        #define ins(_v, _e) oVKeyToEKey.insert(make_pair(_v, CKeyTable::_e))
        // p(26)
        ins('A', eKeyA); ins('B', eKeyB); ins('C', eKeyC); ins('D', eKeyD);
        ins('E', eKeyE); ins('F', eKeyF); ins('G', eKeyG); ins('H', eKeyH);
        ins('I', eKeyI); ins('J', eKeyJ); ins('K', eKeyK); ins('L', eKeyL);
        ins('M', eKeyM); ins('N', eKeyN); ins('O', eKeyO); ins('P', eKeyP);
        ins('Q', eKeyQ); ins('R', eKeyR); ins('S', eKeyS); ins('T', eKeyT);
        ins('U', eKeyU); ins('V', eKeyV); ins('W', eKeyW); ins('X', eKeyX);
        ins('Y', eKeyY); ins('Z', eKeyZ);
        // (10)
        ins('0', eKey0); ins('1', eKey1); ins('2', eKey2); ins('3', eKey3);
        ins('4', eKey4); ins('5', eKey5); ins('6', eKey6); ins('7', eKey7);
        ins('8', eKey8); ins('9', eKey9);
        // t@NV(12)
        ins(VK_F1, eKeyF1); ins(VK_F2, eKeyF2); ins(VK_F3, eKeyF3); ins(VK_F4, eKeyF4);
        ins(VK_F5, eKeyF5); ins(VK_F6, eKeyF6); ins(VK_F7, eKeyF7); ins(VK_F8, eKeyF8);
        ins(VK_F9, eKeyF9); ins(VK_F10, eKeyF10); ins(VK_F11, eKeyF11); ins(VK_F12, eKeyF12);
        // (20)
        ins(VK_ESCAPE, eKeyEsc); ins(VK_PAUSE, eKeyPause); ins(VK_BACK, eKeyBack);
        ins(VK_OEM_PLUS, eKeyPlus); ins(VK_OEM_MINUS, eKeyMinus); ins(VK_TAB, eKeyTab);
        ins(VK_RETURN, eKeyEnter); ins(VK_INSERT, eKeyInsert); ins(VK_DELETE, eKeyDelete);
        ins(VK_HOME, eKeyHome); ins(VK_END, eKeyEnd); ins(VK_PRIOR, eKeyPageUp);
        ins(VK_NEXT, eKeyPageDown); ins(VK_OEM_COMMA, eKeyComma); ins(VK_OEM_PERIOD, eKeyPeriod);
        ins(VK_SPACE, eKeySpace); ins(VK_LEFT, eKeyLeft); ins(VK_UP, eKeyUp);
        ins(VK_RIGHT, eKeyRight); ins(VK_DOWN, eKeyDown);
        // eL[(16)
        ins(VK_NUMPAD0, eKeyNum0); ins(VK_NUMPAD1, eKeyNum1); ins(VK_NUMPAD2, eKeyNum2);
        ins(VK_NUMPAD3, eKeyNum3); ins(VK_NUMPAD4, eKeyNum4); ins(VK_NUMPAD5, eKeyNum5);
        ins(VK_NUMPAD6, eKeyNum6); ins(VK_NUMPAD7, eKeyNum7); ins(VK_NUMPAD8, eKeyNum8);
        ins(VK_NUMPAD9, eKeyNum9); ins(VK_MULTIPLY, eKeyNumMul); ins(VK_ADD, eKeyNumAdd);
        ins(VK_SUBTRACT, eKeyNumSub);
        ins(VK_DIVIDE, eKeyNumDiv);
        #undef ins
        #define ins(_e, _s) oKeyNameList.Insert(CKeyTable::_e, _s);
        // p(26)
        ins(eKeyA, L"A"); ins(eKeyB, L"B"); ins(eKeyC, L"C"); ins(eKeyD, L"D");
        ins(eKeyE, L"E"); ins(eKeyF, L"F"); ins(eKeyG, L"G"); ins(eKeyH, L"H");
        ins(eKeyI, L"I"); ins(eKeyJ, L"J"); ins(eKeyK, L"K"); ins(eKeyL, L"L");
        ins(eKeyM, L"M"); ins(eKeyN, L"N"); ins(eKeyO, L"O"); ins(eKeyP, L"P");
        ins(eKeyQ, L"Q"); ins(eKeyR, L"R"); ins(eKeyS, L"S"); ins(eKeyT, L"T");
        ins(eKeyU, L"U"); ins(eKeyV, L"V"); ins(eKeyW, L"W"); ins(eKeyX, L"X");
        ins(eKeyY, L"Y"); ins(eKeyZ, L"Z");
        // (10)
        ins(eKey1, L"1"); ins(eKey2, L"2"); ins(eKey3, L"3"); ins(eKey4, L"4");
        ins(eKey5, L"5"); ins(eKey6, L"6"); ins(eKey7, L"7"); ins(eKey8, L"8");
        ins(eKey9, L"9"); ins(eKey0, L"0");
        // t@NV(12)
        ins(eKeyF1, L"F1"); ins(eKeyF2, L"F2"); ins(eKeyF3, L"F3");
        ins(eKeyF4, L"F4"); ins(eKeyF5, L"F5"); ins(eKeyF6, L"F6");
        ins(eKeyF7, L"F7"); ins(eKeyF8, L"F8"); ins(eKeyF9, L"F9");
        ins(eKeyF10, L"F10"); ins(eKeyF11, L"F11"); ins(eKeyF12, L"F12");
        // (20)
        ins(eKeyEsc, L"Esc"); ins(eKeyPause, L"Pause"); ins(eKeyBack, L"BackSpace");
        ins(eKeyMinus, L"-"); ins(eKeyPlus, L"+"); ins(eKeyTab, L"Tab");
        ins(eKeyEnter, L"Enter"); ins(eKeyInsert, L"Insert"); ins(eKeyDelete, L"Delete");
        ins(eKeyHome, L"Home"); ins(eKeyEnd, L"End"); ins(eKeyPageUp, L"PageUp");
        ins(eKeyPageDown, L"PageDown"); ins(eKeyComma, L","); ins(eKeyPeriod, L"."); ins(eKeySpace, L"Space");
        ins(eKeyUp, L""); ins(eKeyLeft, L""); ins(eKeyDown, L""); ins(eKeyRight, L"");
        // eL[(16)
        ins(eKeyNum0, L"Num0"); ins(eKeyNum1, L"Num1"); ins(eKeyNum2, L"Num2");
        ins(eKeyNum3, L"Num3"); ins(eKeyNum4, L"Num4"); ins(eKeyNum5, L"Num5");
        ins(eKeyNum6, L"Num6"); ins(eKeyNum7, L"Num7"); ins(eKeyNum8, L"Num8");
        ins(eKeyNum9, L"Num9"); ins(eKeyNumAdd, L"Num +"); ins(eKeyNumSub, L"Num -");
        ins(eKeyNumMul, L"Num *"); ins(eKeyNumDiv, L"Num /");
        // }EX(14)
        ins(eKeyLClick, L"NbN"); ins(eKeyLDouble, L"_uNbN");
        ins(eKeyRClick, L"ENbN"); ins(eKeyRDouble, L"E_uNbN");
        ins(eKeyMClick, L"zC[NbN"); ins(eKeyMDouble, L"zC[_uNbN");
        ins(eKeyMUp, L"zC[()"); ins(eKeyMDown, L"zC[()");
        ins(eKey4Click, L"S{^NbN"); ins(eKey4Double, L"S{^_uNbN");
        ins(eKey5Click, L"T{^NbN"); ins(eKey5Double, L"T{^_uNbN");
        ins(eKeyRLClick, L"ENbN"); ins(eKeyLRClick, L"ENbN");
        #undef ins
    }
};

SKeyTableData& CKeyTable::GetData() {
    static SKeyTableData s_oData;
    return s_oData;
}

string CKeyTable::GetKeyName(EKey eKey) {
    const string* p = GetData().oKeyNameList.FindU(eKey);
    if (p == NULL) {
        Themis_DebugWarning(L"̃L[R[hF" + ItoA((int)eKey));
        return string::Empty;
    }
    return *p;
}

string CKeyTable::GetKeyName(EKey eKey, int nModifiers) {
    StringBuilder str;
    if (nModifiers & eModCtrl) {
        str.Append(L"Ctrl+");
    }
    if (nModifiers & eModAlt) {
        str.Append(L"Alt+");
    }
    if (nModifiers & eModShift) {
        str.Append(L"Shift+");
    }
    if (nModifiers & eModWin) {
        str.Append(L"Win+");
    }
    str.Append(GetKeyName(eKey));
    return str.ToString();
}

pair<CKeyTable::EKey, int> CKeyTable::GetKey(const string& str) {
    int nModifiers = 0;
    int i = 0;
    if (string::Compare(str, i, L"Ctrl+", 0, 5, StringComparison::CurrentCulture) == 0) {
        i += 5;
        nModifiers = eModCtrl;
    }
    if (string::Compare(str, i, L"Alt+", 0, 4, StringComparison::CurrentCulture) == 0) {
        i += 4;
        nModifiers |= eModAlt;
    }
    if (string::Compare(str, i, L"Shift+", 0, 6, StringComparison::CurrentCulture) == 0) {
        i += 6;
        nModifiers |= eModShift;
    }
    if (string::Compare(str, i, L"Win+", 0, 4, StringComparison::CurrentCulture) == 0) {
        i += 4;
        nModifiers |= eModWin;
    }
    const EKey* p = GetData().oKeyNameList.FindT(str.Substring(i));
    if (p == NULL) {
        Themis_DebugWarning(L"`̃L[ۂF" + str);
        return make_pair(EKey(-1), 0);
    }
    return make_pair(*p, nModifiers);
}

pair<UINT, int> CKeyTable::GetVKey(const string& str) {
    pair<EKey, int> p = GetKey(str);
    return pair<UINT, int>(EKeyToVKey(p.first), p.second);
}

CKeyTable::EKey CKeyTable::VKeyToEKey(UINT nVirtualKeyCode) {
    std::map<UINT, EKey>::iterator it = GetData().oVKeyToEKey.find(nVirtualKeyCode);
    if (it != GetData().oVKeyToEKey.end()) {
        return it->second;
    }
    return EKey(-1);
}

UINT CKeyTable::EKeyToVKey(EKey eKey) {
    if (eKey != EKey(-1)) {
        std::map<UINT, EKey>::iterator it = GetData().oVKeyToEKey.begin();
        for (; it != GetData().oVKeyToEKey.end(); ++it) { // 擪珇
            if (it->second == eKey) {
                return it->first;
            }
        }
    }
    return UINT(-1);
}

int CKeyTable::GetModifiers() {
    int n = 0;
    if (::GetKeyState(VK_CONTROL) < 0) { n = eModCtrl; }
    if (::GetKeyState(VK_MENU) < 0) { n |= eModAlt; }
    if (::GetKeyState(VK_SHIFT) < 0) { n |= eModShift; }
    if (::GetKeyState(VK_LWIN) < 0 || ::GetKeyState(VK_RWIN) < 0) {
        n |= eModWin;
    }
    return n;
}

int CKeyTable::GetModifiersAsync() {
    // GetModifiers()RsyāAGetKeyState  GetAsyncKeyState
    int n = 0;
    if (::GetAsyncKeyState(VK_CONTROL) < 0) { n = eModCtrl; }
    if (::GetAsyncKeyState(VK_MENU) < 0) { n |= eModAlt; }
    if (::GetAsyncKeyState(VK_SHIFT) < 0) { n |= eModShift; }
    if (::GetAsyncKeyState(VK_LWIN) < 0 || ::GetAsyncKeyState(VK_RWIN) < 0) {
        n |= eModWin;
    }
    return n;
}

}
