#pragma once
#include "String.h"
#include <boost/noncopyable.hpp>
namespace Themis {

/**
 * .NET̓NXIȂ́B
 *
 * maxCapacity֘A͖B
 */
class StringBuilder : private boost::noncopyable {
    List<wchar_t> buffer;
    int maxCapacity;
public:
    //@{ 
    StringBuilder() : maxCapacity(INT_MAX) {}
    StringBuilder(int capacity) : maxCapacity(INT_MAX) {
        SetCapacity(capacity);
    }
    StringBuilder(const string& value) : maxCapacity(INT_MAX) {
        Append(value);
    }
    StringBuilder(int capacity, int maxCapacity) : maxCapacity(maxCapacity) {
        SetCapacity(capacity);
    }
    StringBuilder(const string& value, int capacity) : maxCapacity(INT_MAX) {
        SetCapacity(capacity);
        Append(value);
    }
    StringBuilder(const string& value, int startIndex, int length, int capacity) : maxCapacity(INT_MAX) {
        SetCapacity(capacity);
        Append(value, startIndex, length);
    }
    //@}

    //@{ e
    int GetCapacity() const { return buffer.GetCapacity(); }
    void SetCapacity(int n) { buffer.SetCapacity(n); }
    //@}
    //@{ TCY
    int GetLength() const { return buffer.GetCount(); }
    void SetLength(int n) { buffer.SetCount(n); }
    //@}
    /// őe
    int MaxCapacity() const { return maxCapacity; }

    //@{ operator[]
    wchar_t& operator[](int index) { return buffer[index]; }
    wchar_t operator[](int index) const { return buffer[index]; }
    //@}

    //@{ ֒ǉ
    StringBuilder& Append(bool value) {
        return Append(value ? L"True" : L"False");
    }
    StringBuilder& Append(char value) {
        return AppendFormat(L"%d", value);
    }
    StringBuilder& Append(short value) {
        return AppendFormat(L"%d", value);
    }
    StringBuilder& Append(int value) {
        return AppendFormat(L"%d", value);
    }
    StringBuilder& Append(long value) {
        return AppendFormat(L"%d", value);
    }
    StringBuilder& Append(float value) {
        return AppendFormat(L"%f", value);
    }
    StringBuilder& Append(double value) {
        return AppendFormat(L"%f", value);
    }
    StringBuilder& Append(unsigned char value) {
        return AppendFormat(L"%u", value);
    }
    StringBuilder& Append(unsigned short value) {
        return AppendFormat(L"%u", value);
    }
    StringBuilder& Append(unsigned int value) {
        return AppendFormat(L"%u", value);
    }
    StringBuilder& Append(unsigned long value) {
        return AppendFormat(L"%u", value);
    }

    StringBuilder& Append(wchar_t value) {
        buffer.Add(value);
        return *this;
    }
    StringBuilder& Append(wchar_t value, int repeatCount) {
        for (int i = 0; i < repeatCount; i++) {
            buffer.Add(value);
        }
        return *this;
    }

    StringBuilder& Append(const wchar_t* value) {
        return Append(value, 0, static_cast<int>(wcslen(value)));
    }
    StringBuilder& Append(const wchar_t* value, int startIndex, int charCount) {
        buffer.insert(buffer.end(), value + startIndex, value + startIndex + charCount);
        return *this;
    }

    StringBuilder& Append(const std::vector<wchar_t>& value) {
        return Append(value, 0, static_cast<int>(value.size()));
    }
    StringBuilder& Append(const std::vector<wchar_t>& value, int startIndex, int charCount) {
        return Append(&value[0], startIndex, charCount);
    }

    StringBuilder& Append(const string& value) {
        return Append(value, 0, value.GetLength());
    }
    StringBuilder& Append(const string& value, int startIndex, int count) {
        return Append(value.ToCharPtr(), startIndex, count);
    }

    StringBuilder& AppendFormat(const wchar_t* format, ...);

    StringBuilder& AppendLine() {
      //return Append(Environment::GetNewLine());
        return Append(L"\r\n");
    }
    StringBuilder& AppendLine(const string& value) {
        return Append(value).AppendLine();
    }
    //@}

  //void CopyTo(int startIndex, const std::vector<wchar_t>& destination, int destinationIndex, int count);
  //int EnsureCapacity(int capacity);
  //bool Equals(StringBuilder sb);

    //@{ wʒuɒǉ
    StringBuilder& Insert(int index, bool value) {
        return Insert(index, value ? L"True" : L"False");
    }
    StringBuilder& Insert(int index, char value) {
        return InsertFormat(index, L"%d", value);
    }
    StringBuilder& Insert(int index, short value) {
        return InsertFormat(index, L"%d", value);
    }
    StringBuilder& Insert(int index, int value) {
        return InsertFormat(index, L"%d", value);
    }
    StringBuilder& Insert(int index, long value) {
        return InsertFormat(index, L"%d", value);
    }
    StringBuilder& Insert(int index, float value) {
        return InsertFormat(index, L"%f", value);
    }
    StringBuilder& Insert(int index, double value) {
        return InsertFormat(index, L"%f", value);
    }
    StringBuilder& Insert(int index, unsigned char value) {
        return InsertFormat(index, L"%u", value);
    }
    StringBuilder& Insert(int index, unsigned short value) {
        return InsertFormat(index, L"%u", value);
    }
    StringBuilder& Insert(int index, unsigned int value) {
        return InsertFormat(index, L"%u", value);
    }
    StringBuilder& Insert(int index, unsigned long value) {
        return InsertFormat(index, L"%u", value);
    }

    StringBuilder& Insert(int index, wchar_t value) {
        buffer.Insert(index, value);
        return *this;
    }
    StringBuilder& Insert(int index, const wchar_t* value) {
        return Insert(index, value, 0, static_cast<int>(wcslen(value)));
    }
    StringBuilder& Insert(int index, const wchar_t* value, int startIndex, int charCount) {
        buffer.insert(buffer.begin() + index, value + startIndex, value + startIndex + charCount);
        return *this;
    }

    StringBuilder& Insert(int index, const std::vector<wchar_t>& value) {
        return Insert(index, value, 0, static_cast<int>(value.size()));
    }
    StringBuilder& Insert(int index, const std::vector<wchar_t>& value, int startIndex, int charCount) {
        return Insert(index, &value[0], startIndex, charCount);
    }

    StringBuilder& Insert(int index, const string& value) {
        return Insert(index, value, value.GetLength());
    }
    StringBuilder& Insert(int index, const string& value, int count) {
        return Insert(index, value.ToCharPtr(), 0, count);
    }
    //@}

    /// 폜
    StringBuilder& Remove(int startIndex, int length) {
        BOOST_ASSERT(0 <= startIndex && 0 <= length && startIndex + length <= GetLength());
        buffer.RemoveRange(startIndex, length);
        return *this;
    }

    //@{ u
    StringBuilder& Replace(wchar_t oldChar, wchar_t newChar) {
        return Replace(oldChar, newChar, 0, GetLength());
    }
    StringBuilder& Replace(const string& oldValue, const string& newValue) {
        return Replace(oldValue, newValue, 0, GetLength());
    }
    StringBuilder& Replace(wchar_t oldChar, wchar_t newChar, int startIndex, int count) {
        BOOST_ASSERT(0 <= startIndex && 0 <= count && startIndex + count <= GetLength());
        if (GetLength() <= 0) return *this;
        std::replace(buffer.begin() + startIndex, 
            buffer.begin() + startIndex + count, oldChar, newChar);
        return *this;
    }
    StringBuilder& Replace(const string& oldValue, const string& newValue, int startIndex, int count) {
        BOOST_ASSERT(0 <= startIndex && 0 <= count && startIndex + count <= GetLength());
        if (GetLength() <= 0) return *this;
        // 蔲BcȂɒxȂc͂B
        string str = ToString(startIndex, count).Replace(oldValue, newValue);
        Remove(startIndex, count);
        return Insert(startIndex, str);
    }
    //@}

    //@{ 
    string ToString() const {
        return buffer;
    }
    string ToString(int startIndex, int length) const {
        BOOST_ASSERT(0 <= startIndex && 0 <= length && startIndex + length <= GetLength());
        return string(&buffer[0], startIndex, length);
    }
    //@}

protected:
    /// ̂.NETɂ͂ꖳ̂ŁApprotectedŁB
    StringBuilder& InsertFormat(int index, const wchar_t* format, ...);

private:
    StringBuilder& Append(const char* value); // ̂boolɈÖق̕ϊĂ܂̂ő΍
};

}
