#include "stdafx.h"
#include "Tooltip.h"
#include "Wnd.h"
#include "WndCtrl.h"
#include "Window.h"
#include "String/Encoding.h"
#include <commctrl.h>
#pragma comment(lib,"comctl32.lib")
namespace Themis {

CDialogTooltip::CDialogTooltip(CWindow& rWindow) {
    Initialize(rWindow.GetHandle(), rWindow.GetHook());
}

CDialogTooltip::CDialogTooltip(HWND hWnd, CMsgHook& rMsgHook) {
    Initialize(hWnd, rMsgHook);
}

CDialogTooltip::~CDialogTooltip() {
    if (m_hToolTip != NULL) {
        HWND hWnd = m_hToolTip;
        m_hToolTip = NULL;
        ::DestroyWindow(hWnd);
    }
}

void CDialogTooltip::Initialize(HWND hWnd, CMsgHook& rMsgHook) {
    ::InitCommonControls();

    m_hWnd = hWnd;
    m_hToolTip = ::CreateWindowEx(0, TOOLTIPS_CLASS, NULL, TTS_ALWAYSTIP | TTS_NOPREFIX,
        CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
        m_hWnd, NULL, ::GetModuleHandle(NULL), NULL);
    Themis_DebugAssertApi(m_hToolTip != NULL);
 #ifndef TTM_SETMAXTIPWIDTH
    const int TTM_SETMAXTIPWIDTH = (WM_USER + 24);
 #endif
    PostMsg(TTM_SETMAXTIPWIDTH, 0, 0x180);

    // ]vȂbȋCftHgŒ߂ɕ\
    SetAutoPopDelay(30 * 1000);

    rMsgHook.Add(this);
}

void CDialogTooltip::Add(int nId, const string& msg) {
    auto_ptr<std::string> p(new std::string(Encoding::GetDefault().GetBytes(msg)));
    InnerAdd(nId, const_cast<char*>(p->c_str()));
    msgs.push_back(p.release());
}

void CDialogTooltip::InnerAdd(int nId, char* pszMsg) {
    if (m_hToolTip != NULL) {
        HWND hCtrl = CWnd(m_hWnd).GetCtrl(nId);
        if (hCtrl != NULL) {
            bool bEdit = CWnd(hCtrl).GetClassName() == L"Edit";
            TOOLINFO ti = {sizeof(TOOLINFO)};
         // ti.hwnd = m_hWnd;
            if (bEdit) {
                // GfBbg{bNXȏꍇA\ʒů֌Wł肷̂ŁBB
                ti.uFlags = TTF_CENTERTIP |TTF_IDISHWND | TTF_SUBCLASS;
            } else {
                ti.uFlags = TTF_IDISHWND | TTF_SUBCLASS;
            }
            ti.uId = reinterpret_cast<UINT_PTR>(hCtrl);
            ti.lpszText = pszMsg; // LPSTR_TEXTCALLBACK;
            SendMsg(TTM_ADDTOOL, 0, (LPARAM)&ti);
        }
    }
}

void CDialogTooltip::Remove(int nId) {
    if (m_hToolTip != NULL) {
        HWND hCtrl = CWnd(m_hWnd).GetCtrl(nId);
        if (hCtrl != NULL) {
            TOOLINFO ti = {sizeof(TOOLINFO)};
            ti.hwnd = m_hWnd;
            ti.uId = reinterpret_cast<UINT_PTR>(hCtrl);
            SendMsg(TTM_DELTOOL, 0, (LPARAM)&ti);
        }
    }
}

/// InitialDelay, AutoPopDelay, ReshowDelayݒ
void CDialogTooltip::SetAutomaticDelay(int n) {
    PostMsg(TTM_SETDELAYTIME, TTDT_AUTOPOP, (LPARAM)n);
}

/// `bvŏɕ\܂ł̎Ԃݒ
void CDialogTooltip::SetInitialDelay(int n) {
    PostMsg(TTM_SETDELAYTIME, TTDT_INITIAL, (LPARAM)n);
}

/// ړƂ̂\܂ł̎Ԃݒ
void CDialogTooltip::SetReshowDelay(int n) {
    PostMsg(TTM_SETDELAYTIME, TTDT_RESHOW, (LPARAM)n);
}

/// `bv\Ă鎞Ԃݒ
void CDialogTooltip::SetAutoPopDelay(int n) {
    PostMsg(TTM_SETDELAYTIME, TTDT_AUTOPOP, (LPARAM)n);
}

bool CDialogTooltip::HookMsg(CMsgHookPrms& rPrms) {
    if (rPrms.GetMsg() == WM_DESTROY) {
        delete this;
        return false;
    } else {
        return false;
    }
}

/*
bool CDialogTooltip::HookMsg(CMsgHookPrms& rPrms) {
    if (m_hToolTip != NULL) {
        if (rPrms.GetMsg() == WM_NOTIFY) {
            TOOLTIPTEXT* tt = (TOOLTIPTEXT*)rPrms.GetLPrm();
            if (tt->hdr.code == TTN_NEEDTEXT) {
                std::map<HWND, string>::iterator it = m_oMsgList.find(reinterpret_cast<HWND>(tt->hdr.idFrom));
                if (it != m_oMsgList.end()) {
                    StrCopy(tt->szText, lengthof(tt->szText), it->second);
                    return true;
                }
            }
            return false;
        }
    }
    return false;
}
*/

}
