#include "stdafx.h"
#include <mmsystem.h>
#pragma comment(lib, "winmm.lib")
#include "VolumeController.h"
namespace Themis {

struct SVolumeControllerData {
    HMIXER                          m_hMixer;
    MIXERCONTROL                    m_mc;
    MIXERCONTROLDETAILS             m_mcd[CVolumeController::c_nDeviceCount];
    MIXERCONTROLDETAILS_UNSIGNED    m_mcdu[CVolumeController::c_nDeviceCount];
    MIXERLINE                       m_ml[CVolumeController::c_nDeviceCount];
    bool                            m_abValid[CVolumeController::c_nDeviceCount];

    bool GetMixerLineControl(CVolumeController::EDevice e, const MIXERLINE& ml) {
        vector<MIXERCONTROL> mc(ml.cControls);
        MIXERLINECONTROLS mlc = {
            sizeof(MIXERLINECONTROLS),
            ml.dwLineID,
            0,
            ml.cControls,
            sizeof(MIXERCONTROL),
            &mc[0]
        };
        MMRESULT res = ::mixerGetLineControls((HMIXEROBJ)m_hMixer, &mlc, MIXER_GETLINECONTROLSF_ALL);
        if (res != MMSYSERR_NOERROR) {
            return false;
        }
        for (DWORD i = 0; i < mlc.cControls; i++) {
            if (mc[i].dwControlType == MIXERCONTROL_CONTROLTYPE_VOLUME) {
                // {[Rg[\ۂꍇ
                m_mcd[e].cbStruct = sizeof(MIXERCONTROLDETAILS);
                m_mcd[e].dwControlID = mc[i].dwControlID;
                m_mcd[e].cChannels = 1;
                m_mcd[e].cMultipleItems = mc[i].cMultipleItems;
                m_mcd[e].cbDetails = sizeof(MIXERCONTROLDETAILS_UNSIGNED);
                m_mcd[e].paDetails = &m_mcdu[e];
                res = ::mixerGetControlDetails((HMIXEROBJ)m_hMixer, &m_mcd[e], 0);
                if (res == MMSYSERR_NOERROR) {
                    memcpy(&m_mc, &mc[0], sizeof(MIXERCONTROL));
                    return true;
                }
            }
        }
        return false;
    }
};

CVolumeController::CVolumeController() {
    try {
        data->m_hMixer = NULL;
        std::fill(data->m_abValid, data->m_abValid + c_nDeviceCount, false);
        // foCXJ
        if (!Open()) {
            throw RuntimeException(L"TEh~LT[̏Ɏs܂");
        }
        // foCX̎擾
        MIXERCAPS mc = {0};
        MMRESULT res = ::mixerGetDevCaps(0, &mc, sizeof(mc));
        if (res != MMSYSERR_NOERROR) {
            throw RuntimeException(L"TEh~LT[̏Ɏs܂");
        }
        for (DWORD i = 0; i < mc.cDestinations; i++) {
            MIXERLINE ml = {sizeof(ml), i};
            res = ::mixerGetLineInfo((HMIXEROBJ)data->m_hMixer, &ml, MIXER_GETLINEINFOF_DESTINATION);
            if (res != MMSYSERR_NOERROR) {
                continue;
            }
            if (ml.dwComponentType != MIXERLINE_COMPONENTTYPE_DST_SPEAKERS) {
                continue;
            }
            if (!data->GetMixerLineControl(eDeviceMaster, ml)) {
                continue;
            }
            data->m_ml[eDeviceMaster] = ml;
            data->m_abValid[eDeviceMaster] = true;
            // efoCX̃\[X̎擾
            for (DWORD j = 0, nConnections = ml.cConnections; j < nConnections; j++) {
                ml.cbStruct = sizeof(ml);
                ml.dwDestination = i;
                ml.dwSource = j;
                res = ::mixerGetLineInfo((HMIXEROBJ)data->m_hMixer, &ml, MIXER_GETLINEINFOF_SOURCE);
                if (res != MMSYSERR_NOERROR) {
                    continue;
                }
                switch (ml.dwComponentType) {
                case MIXERLINE_COMPONENTTYPE_SRC_WAVEOUT:
                    if (data->GetMixerLineControl(eDeviceWave, ml)) {
                        data->m_ml[eDeviceWave] = ml;
                        data->m_abValid[eDeviceWave] = true;
                    }
                    break;
                case MIXERLINE_COMPONENTTYPE_SRC_SYNTHESIZER:
                    if (data->GetMixerLineControl(eDeviceMidi, ml)) {
                        data->m_ml[eDeviceMidi] = ml;
                        data->m_abValid[eDeviceMidi] = true;
                    }
                    break;
                case MIXERLINE_COMPONENTTYPE_SRC_COMPACTDISC:
                    if (data->GetMixerLineControl(eDeviceCd, ml)) {
                        data->m_ml[eDeviceCd] = ml;
                        data->m_abValid[eDeviceCd] = true;
                    }
                    break;
                case MIXERLINE_COMPONENTTYPE_SRC_LINE:  //MIXERLINE_COMPONENTTYPE_SRC_AUXILIARY ?
                    if (data->GetMixerLineControl(eDeviceLine, ml)) {
                        data->m_ml[eDeviceLine] = ml;
                        data->m_abValid[eDeviceLine] = true;
                    }
                    break;
                }
            }
        }
    } catch (std::exception&) {
        Themis_DebugWarning(L"TEh~LT[̏Ɏs܂");
        Close();
    }
}

CVolumeController::~CVolumeController() {
    Close();
}

int CVolumeController::GetVolume(EDevice eDevice) const {
    return (data->m_mcdu[eDevice].dwValue * 100 + 65535/2) / 65535;  //ľܓƂ
}

void CVolumeController::SetVolume(EDevice eDevice, int nPercent) {
    DWORD dwVolume;
    if (nPercent < 0) {
        dwVolume = 0;
    } else if (100 <= nPercent) {
        dwVolume = 65535;
    } else {
        dwVolume = (65535 * nPercent + 50) / 100;   //ľܓƂ
    }

    data->m_mcdu[eDevice].dwValue = dwVolume;
    MMRESULT res = ::mixerSetControlDetails((HMIXEROBJ)data->m_hMixer, &data->m_mcd[eDevice], 0);
    Themis_DebugAssert(res == MMSYSERR_NOERROR);
}

bool CVolumeController::Open() {
    // foCX̃I[v
    if (::mixerGetNumDevs() == 0) {
        return false;
    }
    return ::mixerOpen(&data->m_hMixer, 0, 0, 0, MIXER_OBJECTF_MIXER) == MMSYSERR_NOERROR;
}

void CVolumeController::Close() {
    if (data->m_hMixer != NULL) {
        MMRESULT res = ::mixerClose(data->m_hMixer);
        Themis_DebugAssert(res == MMSYSERR_NOERROR);
        data->m_hMixer = NULL;
    }
}

}
