#pragma once
namespace Themis {

namespace ProcessPriorityClass {
    enum T {
        RealTime = REALTIME_PRIORITY_CLASS,
        High = HIGH_PRIORITY_CLASS,
        AboveNormal = ABOVE_NORMAL_PRIORITY_CLASS,
        Normal = NORMAL_PRIORITY_CLASS,
        BelowNormal = BELOW_NORMAL_PRIORITY_CLASS,
        Idle = IDLE_PRIORITY_CLASS,
    };
}

namespace ProcessWindowStyle {
    enum T {
        Normal = SW_SHOWNORMAL,
        Hidden = SW_HIDE,
        Minimizedw = SW_SHOWMINIMIZED,
        Maximized = SW_SHOWMAXIMIZED,
        NoActivate = SW_SHOWNOACTIVATE,             // ANeBu
        MinimizedNoActivate = SW_SHOWMINNOACTIVE,    // ŏANeBu
    };
}

/**
 * Process::Start()pp[^
 */
struct ProcessStartInfo {
    string FileName;
    string Arguments;
    string Verb;
    string WorkingDirectory;
    bool UseShellExecute;
    bool CreateNoWindow;
    bool ErrorDialog;
    HWND ErrorDialogParentHandle;
    //string Domain;
    //bool LoadUserProfile;
    //string UserName;
    //SecureString Password;
    //bool RedirectStandardError;
    //bool RedirectStandardInput;
    //bool RedirectStandardOutput;
    //Encoding StandardErrorEncoding;
    //Encoding StandardOutputEncoding;
    ProcessWindowStyle::T WindowStyle;

    ProcessStartInfo() {
        Initialize();
    }
    ProcessStartInfo(string fileName) {
        Initialize();
        FileName = fileName;
    }
    ProcessStartInfo(string fileName, string arguments) {
        Initialize();
        FileName = fileName;
        Arguments = arguments;
    }

    //std::map<string, string> GetEnvironmentVariables() const;
    //List<string> GetVerbs() const;

private:
    void Initialize() {
        UseShellExecute = true;
        CreateNoWindow = false;
        ErrorDialog = false;
        ErrorDialogParentHandle = NULL;
        //LoadUserProfile = false;
        //RedirectStandardError = false;
        //RedirectStandardInput = false;
        //RedirectStandardOutput = false;
        //StandardErrorEncoding = NULL;
        //StandardOutputEncoding = NULL;
        WindowStyle = ProcessWindowStyle::Normal;
    }
};

/**
 * vZXB
 */
class Process : private boost::noncopyable {
    ProcessStartInfo startInfo;
    HANDLE processHandle;
protected:
    Process(HANDLE processHandle) : processHandle(processHandle) {}
public:
    Process() : processHandle(INVALID_HANDLE_VALUE) {}
    ~Process() { Close(); }

    /// ݂̃vZX̎擾
    static std::tr1::shared_ptr<Process> GetCurrentProcess() {
        return std::tr1::shared_ptr<Process>(new Process(::GetCurrentProcess()));
    }
    
    /// nhB(vZXI킯ł͂Ȃ)
    void Close();

    /// ProcessStartInfo
    const ProcessStartInfo& GetStartInfo() const { return startInfo; }
    /// ProcessStartInfo
    void SetStartInfo(const ProcessStartInfo& value) { startInfo = value; }

    /// vZXDx
    ProcessPriorityClass::T GetPriorityClass() const {
        DWORD priorityClass = ::GetPriorityClass(processHandle);
        Themis_DebugAssertApi(priorityClass == 0);
        return (ProcessPriorityClass::T)priorityClass;
    }
    /// vZXDx
    void SetPriorityClass(ProcessPriorityClass::T value) {
        BOOL ret = ::SetPriorityClass(processHandle, (DWORD)value);
        Themis_DebugAssertApi(ret != FALSE);
    }

    /// vZX̊Jn
    static std::tr1::shared_ptr<Process> Start(string fileName) {
        return Start(ProcessStartInfo(fileName));
    }
    /// vZX̊Jn
    static std::tr1::shared_ptr<Process> Start(string fileName, string arguments) {
        return Start(ProcessStartInfo(fileName, arguments));
    }
    /// vZX̊Jn
    static std::tr1::shared_ptr<Process> Start(const ProcessStartInfo& startInfo) {
        std::tr1::shared_ptr<Process> process(new Process());
        process->SetStartInfo(startInfo);
        if (process->Start()) {
            return process;
        }
        return std::tr1::shared_ptr<Process>();
    }
    /// vZX̊Jn
    bool Start() {
        Close();
        ProcessStartInfo startInfo = GetStartInfo();
        Themis_DebugAssert(!string::IsNullOrEmpty(startInfo.FileName));
        return startInfo.UseShellExecute ?
            StartWithShellExecuteEx(startInfo) :
            StartWithCreateProcess(startInfo);
    }

    /// vZXIԂɂȂ܂ő҂
    void WaitForExit() {
        WaitForExit(INFINITE);
    }
    /// vZXIԂɂȂ܂ő҂
    bool WaitForExit(int milliseconds) {
        switch (::WaitForSingleObject(processHandle, milliseconds)) {
        case WAIT_OBJECT_0: return true;
        case WAIT_TIMEOUT: return false;
        default:
            Themis_DebugApiError();
            return false;
        }
    }
    /// vZXAChԂɂȂ܂ő҂
    bool WaitForInputIdle() {
        return WaitForInputIdle(INFINITE);
    }
    /// vZXAChԂɂȂ܂ő҂
    bool WaitForInputIdle(int milliseconds) {
        switch (::WaitForInputIdle(processHandle, milliseconds)) {
        case 0: return true;
        case WAIT_TIMEOUT: return false;
        default:
            Themis_DebugApiError();
            return false;
        }
    }

private:
    bool StartWithCreateProcess(const ProcessStartInfo& info);
    bool StartWithShellExecuteEx(const ProcessStartInfo& info);
    static string BuildCommandLine(const string& executableFileName, const string& arguments);
};

}
