#include "stdafx.h"
#include "Wnd.h"
#include "WndCtrl.h"
#include "String/Encoding.h"
#include "Utility/Algorithm.h"
#include "Utility/StringUtility.h"
#include <commctrl.h>
#pragma comment(lib, "comctl32.lib")
namespace Themis {

bool CCtrl::Exists() { return CWnd(m_hWnd).Exists(); }
DWORD CCtrl::GetStyle() { return CWnd(m_hWnd).GetStyle(); }
DWORD CCtrl::GetExStyle() { return CWnd(m_hWnd).GetExStyle(); }
CRect CCtrl::GetRect() { return CWnd(m_hWnd).GetRect(); }
CPoint CCtrl::GetPos() { return CWnd(m_hWnd).GetPos(); }
CSize CCtrl::GetSize() { return CWnd(m_hWnd).GetSize(); }
CRect CCtrl::GetClientRect() { return CWnd(GetHandle()).GetClientRect(); }
CPoint CCtrl::GetClientPos() { return CWnd(GetHandle()).GetClientPos(); }
CSize CCtrl::GetClientSize() { return CWnd(GetHandle()).GetClientSize(); }
bool CCtrl::IsEnabled() { return CWnd(m_hWnd).IsEnabled(); }
bool CCtrl::IsVisible() { return CWnd(m_hWnd).IsVisible(); }
bool CCtrl::IsVisibleHScroll() const { return CWnd(m_hWnd).IsVisibleHScroll(); }
bool CCtrl::IsVisibleVScroll() const { return CWnd(m_hWnd).IsVisibleVScroll(); }
CSize CCtrl::GetValidClientSize() const { return CWnd(m_hWnd).GetValidClientSize(); }
void CCtrl::SetStyle(DWORD dw) { CWnd(m_hWnd).SetStyle(dw); }
void CCtrl::SetExStyle(DWORD dw) { CWnd(m_hWnd).SetExStyle(dw); }
void CCtrl::SetRect(const CRect& rc) { CWnd(m_hWnd).SetRect(rc); }
void CCtrl::SetPos(const CPoint& pt) { CWnd(m_hWnd).SetPos(pt); }
void CCtrl::SetSize(const CSize& sz) { CWnd(m_hWnd).SetSize(sz); }
void CCtrl::SetClientSize(const CSize& sz) { CWnd(GetHandle()).SetClientSize(sz); }
void CCtrl::SetEnable(bool b) { CWnd(m_hWnd).SetEnabled(b); }
void CCtrl::ReverseEnabled() { SetEnable(!IsEnabled()); }
void CCtrl::Show(int n) { CWnd(m_hWnd).Show(n); }
void CCtrl::Hide() { Show(SW_HIDE); }
void CCtrl::SetFocus() { CWnd(m_hWnd).SetFocus(); }
void CCtrl::Update(){ CWnd(m_hWnd).Update(); }
void CCtrl::PostUpdate() { CWnd(m_hWnd).PostUpdate(); }

string CCtrl::GetText() const { return CWnd(m_hWnd).GetText(); }
void CCtrl::SetText(const string& text) { CWnd(m_hWnd).SetText(text); }
int CCtrl::GetInt() const { return AtoI(GetText()); }
void CCtrl::SetInt(int n) { SetText(ItoA(n)); }

LRESULT CCtrl::SendMsg(UINT u, WPARAM w, LPARAM l) const { return CWnd(m_hWnd).SendMsg(u, w, l); }
void CCtrl::PostMsg(UINT u, WPARAM w, LPARAM l) const { CWnd(m_hWnd).PostMsg(u, w, l); }
EWndCtrlType CCtrl::GetCtrlType() const { return CWnd(m_hWnd).GetCtrlType(); }

//------------------------------------------------------------------//

bool CCheckbox::GetCheck() const {
    return (BST_CHECKED == SendMsg(BM_GETCHECK, 0, 0));
}

void CCheckbox::SetCheck(bool b) {
    SendMsg(BM_SETCHECK, static_cast<WPARAM>(b? BST_CHECKED: BST_UNCHECKED), 0);
}

//------------------------------------------------------------------//

/// Ȃtrue
bool CEditbox::IsEmpty() const {
    return ::GetWindowTextLength(GetHandle()) <= 0;
}

/// ύXĂtrue
bool CEditbox::IsModified() const {
    return SendMsg(EM_GETMODIFY, 0, 0) != FALSE;
}

/// ύXĂ邩ǂtO蓮ݒ
CEditbox& CEditbox::SetModified(bool b) {
    SendMsg(EM_SETMODIFY, b ? TRUE : FALSE, 0);
    return *this;
}

/// pX[hȓ͗ɂ߂
CEditbox& CEditbox::SetPasswordMode(bool b) {
    if (b) {
        SetStyle(GetStyle() | ES_PASSWORD);
        SendMsg(EM_SETPASSWORDCHAR, L'*', 0);
    } else {
        SetStyle(GetStyle() & ~ES_PASSWORD);
        SendMsg(EM_SETPASSWORDCHAR, 0, 0);
    }
    Update();
    return *this;
}

CEditbox& CEditbox::InsertText(const string& strText) {
    SendMsg(EM_REPLACESEL, 0, (LPARAM)Encoding::GetDefault().GetBytes(strText).c_str());
    return *this;
}

CEditbox& CEditbox::SetSelected(int nBegin, int nEnd) {
    SendMsg(EM_SETSEL, (WPARAM)nBegin, (LPARAM)nEnd);
    return *this;
}

//------------------------------------------------------------------//

string CListbox::Get(int n) const {
    size_t nLen = SendMsg(LB_GETTEXTLEN, n, 0) + 1;
    if (nLen <= 0) {
        return string();
    }
    boost::scoped_array<char> sz(new char[nLen]);
    SendMsg(LB_GETTEXT, n, reinterpret_cast<LPARAM>(sz.get()));
    return Encoding::GetDefault().GetString(sz.get());
}

string CListbox::Get() const {
    return Get(GetSelected());
}

vector<string> CListbox::GetAll() const {
    vector<string> vct(GetCount());
    for (size_t i = 0, n = vct.size(); i < n; i++) {
        vct[i] = Get(static_cast<int>(i));
    }
    return vct;
}

int CListbox::GetCount() const {
    return static_cast<int>(SendMsg(LB_GETCOUNT, 0, 0));
}

int CListbox::GetSelected() const {
    Themis_DebugAssert(LB_ERR == -1);
    return static_cast<int>(SendMsg(LB_GETCURSEL, 0, 0));
}

CListbox& CListbox::Add(const string& strText) {
    // return static_cast<int>(SendMsg(`)); Ă̂AǁBB
    SendMsg(LB_ADDSTRING, 0, reinterpret_cast<LPARAM>(Encoding::GetDefault().GetBytes(strText).c_str()));
    return *this;
}

CListbox& CListbox::Add(const vector<string>& vct) {
    for (size_t i = 0, n = vct.size(); i < n; i++) {
        Add(vct[i]);
    }
    return *this;
}

CListbox& CListbox::Insert(int n, const string& strText) {
    SendMsg(LB_INSERTSTRING, n, reinterpret_cast<LPARAM>(Encoding::GetDefault().GetBytes(strText).c_str()));
    return *this;
}

CListbox& CListbox::Delete(int n) {
    SendMsg(LB_DELETESTRING, n, 0);
    return *this;
}

/// ݂̍ڂ폜
CListbox& CListbox::Delete() {
    int n = GetSelected();
    Delete(n);
    SetSelected(Min(n, GetCount() - 1));
    return *this;
}

CListbox& CListbox::Clear() {
    int nSize = GetCount();
    for (int i=0; i<nSize; i++) {
        Delete(nSize - 1 - i);  //납Ă
    }
    return *this;
}

CListbox& CListbox::SetSelected(int nIndex) {
    SendMsg(LB_SETCURSEL, nIndex, 0);
    return *this;
}

CListbox& CListbox::SetSelected(const string& str) {
    SetSelected(Find(str));
    return *this;
}

/// ďԂBȂ-1
int CListbox::Find(const string& str) const {
    for (int i = 0, n = GetCount(); i < n; i++) {
        if (Get(i) == str) {
            return i;
        }
    }
    return -1;
}

/// ڂɈړ
bool CListbox::Up(int nIndex, bool bSelect) {
    if (0 < nIndex && nIndex < GetCount()) {
        string str = Get(nIndex);
        Delete(nIndex);
        Insert(nIndex - 1, str);
        SetSelected(nIndex - 1);
        return true;
    }
    return false;
}

/// ڂɈړ
bool CListbox::Down(int nIndex, bool bSelect) {
    if (0 <= nIndex && nIndex + 1 < GetCount()) {
        string str = Get(nIndex);
        Delete(nIndex);
        Insert(nIndex + 1, str);
        SetSelected(nIndex + 1);
        return true;
    }
    return false;
}

//------------------------------------------------------------------//

string CCombobox::Get(int n) const {
    size_t nLen = SendMsg(CB_GETLBTEXTLEN, static_cast<WPARAM>(n), 0) + 1;
    if (nLen <= 0) {
        return string();
    }
    boost::scoped_array<char> sz(new char[nLen]);
    SendMsg(CB_GETLBTEXT, static_cast<WPARAM>(n), reinterpret_cast<LPARAM>(sz.get()));
    return Encoding::GetDefault().GetString(sz.get());
}

string CCombobox::Get() const {
    return Get(GetSelected());
}

int CCombobox::GetCount() const {
    return static_cast<int>(SendMsg(CB_GETCOUNT, 0, 0));
}

int CCombobox::GetSelected() const {
    Themis_DebugAssert(CB_ERR == -1);
    return static_cast<int>(SendMsg(CB_GETCURSEL, 0, 0));
}

CCombobox& CCombobox::Add(const string& str) {
    Insert(-1, str);
    return *this;
}

CCombobox& CCombobox::Clear() {
    SendMsg(CB_RESETCONTENT, 0, 0);
    return *this;
}

CCombobox& CCombobox::Add(const vector<string>& vct) {
    for (size_t i = 0, n = vct.size(); i < n; i++) {
        Add(vct[i]);
    }
    return *this;
}

CCombobox& CCombobox::Insert(int n, const string& str) {
    SendMsg(CB_INSERTSTRING, static_cast<WPARAM>(n), reinterpret_cast<LPARAM>(Encoding::GetDefault().GetBytes(str).c_str()));
    return *this;
}

CCombobox& CCombobox::Delete(int n) {
    SendMsg(CB_DELETESTRING, static_cast<WPARAM>(n), 0);
    return *this;
}

void CCombobox::SetSelected(int n) {
    SendMsg(CB_SETCURSEL, static_cast<WPARAM>(n), 0);
}

CCombobox& CCombobox::SetSelected(const string& str) {
    SendMsg(CB_SELECTSTRING, 0, reinterpret_cast<LPARAM>(Encoding::GetDefault().GetBytes(str).c_str()));
    return *this;
}

//------------------------------------------------------------------//

int CUpdown::Get() const {
    return static_cast<int>(SendMsg(UDM_GETPOS, 0, 0));
}

CUpdown& CUpdown::SetBuddy(HWND hWnd) {
    SendMsg(UDM_SETBUDDY, WPARAM(hWnd), 0);
    return *this;
}

CUpdown& CUpdown::SetRange(int nMin, int nMax) {
    SendMsg(UDM_SETRANGE, 0, MAKELPARAM(nMin, nMax));
    return *this;
}

CUpdown& CUpdown::SetStep(int nStep) {
    UDACCEL s = {0, nStep};
    SendMsg(UDM_SETACCEL, 1, LPARAM(&s));
    return *this;
}

void CUpdown::Set(int n) {
    SendMsg(UDM_SETPOS, TRUE, n);
}

//------------------------------------------------------------------//

int CSlider::Get() const {
    return static_cast<int>(SendMsg(TBM_GETPOS, 0, 0));
}

CSlider& CSlider::SetRange(int nMin, int nMax) {
    SendMsg(TBM_SETRANGE, 0, MAKELPARAM(nMin, nMax));
    return *this;
}

CSlider& CSlider::SetTicFreq(int nFreq) {
    SendMsg(TBM_SETTICFREQ, nFreq, 0);
    return *this;
}

CSlider& CSlider::SetPageSize(int nSize) {
    SendMsg(TBM_SETPAGESIZE, 0, nSize);
    return *this;
}

CSlider& CSlider::SetLineSize(int nSize) {
    SendMsg(TBM_SETLINESIZE, 0, nSize);
    return *this;
}

void CSlider::Set(int n) {
    SendMsg(TBM_SETPOS, TRUE, n);
}

//------------------------------------------------------------------//

CProgress& CProgress::SetRange(int nMin, int nMax) {
    PostMsg(PBM_SETRANGE, 0, MAKELPARAM(nMin, nMax));
    return *this;
}

CProgress& CProgress::SetStep(int n) {
    PostMsg(PBM_SETSTEP, static_cast<WPARAM>(n), 0);
    return *this;
}

CProgress& CProgress::Step() {
    PostMsg(PBM_STEPIT, 0, 0);
    return *this;
}

CProgress& CProgress::Set(int n) {
    PostMsg(PBM_SETPOS, static_cast<WPARAM>(n), 0);
    return *this;
}

//------------------------------------------------------------------//

void CListview::SetExListStyle(DWORD dwExStyle) {
    //LVM_SETEXTENDEDLISTVIEWSTYLE
    SendMsg(0x1036, 0, dwExStyle);
}

void CListview::SetExListStyleMasked(DWORD dwExStyle, DWORD dwMask) {
    //LVM_SETEXTENDEDLISTVIEWSTYLE
    SendMsg(0x1036, dwMask, dwExStyle);
}

DWORD CListview::GetExStyle() const {
    //LVM_GETEXTENDEDLISTVIEWSTYLE
    return static_cast<DWORD>(SendMsg(0x1037, 0, 0));
}

void CListview::RemoveColumn(int nIndex) {
    ListView_DeleteColumn(GetHandle(), nIndex);
}

void CListview::AddColumn(const string& str, int nWidth, int nPos) {
    AddColumn(GetColumnCount(), str, nWidth, nPos);
}

void CListview::AddColumn(int nIndex, const string& text, int nWidth, int nPos) {
    std::string str = Encoding::GetDefault().GetBytes(text);
    LVCOLUMN c = {0};
    c.iSubItem = nIndex;
    c.mask = LVCF_FMT | LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM;
    switch (nPos) {
    case 0: c.fmt = LVCFMT_LEFT; break;
    case 1: c.fmt = LVCFMT_CENTER; break;
    case 2: c.fmt = LVCFMT_RIGHT; break;
    default: Themis_DebugTrespass(); break;
    }
    c.cx = nWidth;
    c.pszText = const_cast<char*>(str.c_str());
    ListView_InsertColumn(GetHandle(), nIndex, &c);
}
void CListview::SetColumn(int nIndex, const string& text, int nWidth, int nPos) {
    std::string str = Encoding::GetDefault().GetBytes(text);
    LVCOLUMN c = {0};
    c.iSubItem = nIndex;
    c.mask = LVCF_FMT | LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM;
    switch (nPos) {
    case 0: c.fmt = LVCFMT_LEFT; break;
    case 1: c.fmt = LVCFMT_CENTER; break;
    case 2: c.fmt = LVCFMT_RIGHT; break;
    default: Themis_DebugTrespass(); break;
    }
    c.cx = nWidth;
    c.pszText = const_cast<char*>(str.c_str());
    ListView_SetColumn(GetHandle(), nIndex, &c);
}

void CListview::SetColumnWidth(int nIndex, int nWidth) {
    LVCOLUMN c = {0};
    c.iSubItem = nIndex;
    c.mask = LVCF_WIDTH;
    c.cx = nWidth;
    ListView_SetColumn(GetHandle(), nIndex, &c);
}

int CListview::GetColumnWidth(int nIndex) const {
    return ListView_GetColumnWidth(GetHandle(), nIndex);
}

string CListview::GetColumnText(int nIndex) const {
    char sz[0x200];
    LVCOLUMN c = {0};
    c.mask = LVCF_TEXT;
    c.pszText = sz;
    c.cchTextMax = sizeof(sz);
    if (ListView_GetColumn(GetHandle(), nIndex, &c) != FALSE) {
        return Encoding::GetDefault().GetString(sz);
    }
    return string::Empty;
}

int CListview::GetColumnCount() const {
    HWND hHeader = ListView_GetHeader(GetHandle());
    if (hHeader != NULL) {
        return Header_GetItemCount(hHeader);
    }
    return -1;
}

void CListview::SetNormalImageList(void* hImageList) {
    ListView_SetImageList(GetHandle(), (HIMAGELIST)hImageList, LVSIL_NORMAL);
}

void CListview::SetSmallImageList(void* hImageList) {
    ListView_SetImageList(GetHandle(), (HIMAGELIST)hImageList, LVSIL_SMALL);
}

void CListview::SetStateImageList(void* hImageList) {
    ListView_SetImageList(GetHandle(), (HIMAGELIST)hImageList, LVSIL_STATE);
}

void CListview::Clear() {
    ListView_DeleteAllItems(GetHandle());
}

void CListview::AddItems(const vector<string>& vct) {
    for (size_t i = 0, n = vct.size(); i < n; i++) {
        AddItem(vct[i]);
    }
}

int CListview::AddItem(const string& str, int nIconIndex) {
 // AddItem(INT_MAX, str, nIconIndex);
    int nIndex = GetCount();
    AddItem(nIndex, str, nIconIndex);
    return nIndex;
}

void CListview::AddItem(int nIndex, const string& text, int nIconIndex) {
    std::string str = Encoding::GetDefault().GetBytes(text);
    LVITEM i = {0};
    i.iItem = nIndex;
    i.mask = LVIF_TEXT;
    i.pszText = const_cast<char*>(str.c_str());
    // ACRȂǉ
    if (0 <= nIconIndex) {
        i.mask |= LVIF_IMAGE;
        i.iImage = nIconIndex;
    }
/*ListView_SortItemsEx()gꍇ͗vȂۂ
    // \[gKvȏꍇnIndexlParamցB
    if ((GetStyle() & LVS_NOSORTHEADER) == 0) {
        i.mask |= LVIF_PARAM;
        i.lParam = nIndex;
    }
*/
    ListView_InsertItem(GetHandle(), &i);
}

void CListview::SetSubItem(int nIndex, int nSubIndex, const string& text) {
    std::string str = Encoding::GetDefault().GetBytes(text);
    LVITEM i = {0};
    i.iItem = nIndex;
    i.iSubItem = nSubIndex;
    i.mask = LVIF_TEXT;
    i.pszText = const_cast<char*>(str.c_str());
    ListView_SetItem(GetHandle(), &i);
}

void CListview::SetIcon(int nIndex, int nIconIndex) {
    LVITEM i = {0};
    i.iItem = nIndex;
    i.mask = LVIF_IMAGE;
    i.iImage = nIconIndex;
    ListView_SetItem(GetHandle(), &i);
}

string CListview::GetSubItemText(int nIndex, int nSubIndex) const {
    char sz[0x400];
    ListView_GetItemText(GetHandle(), nIndex, nSubIndex, sz, sizeof(sz));
    return Encoding::GetDefault().GetString(sz);
}

/// \[gBLVN_COLUMNCLICKĂԂׂB
void CListview::SortItems(FnCompare fnCompare, LPARAM lParamSort) {
    // ListView_SortItems()LPARAMʓ|Ȃ̂ŁBB
    if (ListView_SortItemsEx(GetHandle(), fnCompare, lParamSort) == FALSE) {
        Themis_DebugWarning(L"\[gɎsBr֐Ƃ~XĂH");
    }
}

namespace {
    const int HDF_SORTDOWN = 0x0200;
    const int HDF_SORTUP   = 0x0400;
}

void CListview::SetColumnSortUp(int nSubIndex) {
    HWND hHeader = ListView_GetHeader(GetHandle());
    if (hHeader != NULL) {
        HDITEM i = {0};
        i.mask = HDI_FORMAT;
        if (Header_GetItem(hHeader, nSubIndex, &i) != FALSE) {
            i.fmt &= ~HDF_SORTDOWN;
            i.fmt |= HDF_SORTUP;
            Header_SetItem(hHeader, nSubIndex, &i);
        }
    }
}

void CListview::SetColumnSortDown(int nSubIndex) {
    HWND hHeader = ListView_GetHeader(GetHandle());
    if (hHeader != NULL) {
        HDITEM i = {0};
        i.mask = HDI_FORMAT;
        if (Header_GetItem(hHeader, nSubIndex, &i) != FALSE) {
            i.fmt &= ~HDF_SORTUP;
            i.fmt |= HDF_SORTDOWN;
            Header_SetItem(hHeader, nSubIndex, &i);
        }
    }
}

void CListview::ResetSetColumnSort(int nSubIndex) {
    HWND hHeader = ListView_GetHeader(GetHandle());
    if (hHeader != NULL) {
        HDITEM i = {0};
        i.mask = HDI_FORMAT;
        if (Header_GetItem(hHeader, nSubIndex, &i) != FALSE) {
            i.fmt &= ~HDF_SORTUP;
            i.fmt &= ~HDF_SORTDOWN;
            Header_SetItem(hHeader, nSubIndex, &i);
        }
    }
}

void CListview::ResetSetColumnsSort() {
    for (int i = 0, n = GetColumnCount(); i < n; i++) {
        ResetSetColumnSort(i);
    }
}

void CListview::SetItemCount(int nCount) {
    ListView_SetItemCount(GetHandle(), nCount);
}

void CListview::UpdateItem(int nIndex) {
    ListView_Update(GetHandle(), nIndex);
}

void CListview::UpdateItems() {
    ListView_SetItemCount(GetHandle(), ListView_GetItemCount(GetHandle()));
}

void CListview::Remove(int nIndex) {
    ListView_DeleteItem(GetHandle(), nIndex);
}

int CListview::GetCount() const {
    return static_cast<int>(ListView_GetItemCount(GetHandle()));
}

namespace {
    inline bool IsSelectedListviewItem(HWND hList, int nIndex) {
     // return ListView_GetItemState(hList, nIndex, LVIS_SELECTED | LVIS_FOCUSED) != 0;
        return ListView_GetItemState(hList, nIndex, LVIS_SELECTED) != 0;
    }
    inline void SetSelectedListviewItem(HWND hList, int nIndex, bool bSelected) {
        ListView_SetItemState(hList, nIndex, bSelected? (LVIS_SELECTED | LVIS_FOCUSED): 0, LVIS_SELECTED | LVIS_FOCUSED);
    }
}

void CListview::SetSelected(int nIndex, bool bSelected) {
    SetSelectedListviewItem(GetHandle(), nIndex, bSelected);
}

/// Ԗڂ̍ڂIĂ̂
int CListview::GetSelected() const {
    for (int i = 0, n = GetCount(); i < n; i++) {
        if (IsSelectedListviewItem(GetHandle(), i)) {
            return i;
        }
    }
    return -1;
}

/// IĂ鍀ڂ̃XgԂ
vector<int> CListview::GetSelectedIndexes() const {
    vector<int> vct;
    for (int i = -1;;) {
        int n = ListView_GetNextItem(GetHandle(), i, LVNI_ALL | LVNI_SELECTED);
        if (n < 0) {
            break;
        }
        vct.push_back(n);
        i = n;
    }
    return vct;
}

/// nIndexԖڂ̍ڂIĂ̂ǂ擾
bool CListview::IsSelected(int nIndex) {
    return IsSelectedListviewItem(GetHandle(), nIndex);
}

/// SĂI
void CListview::SetSelectAll(bool bSelected) {
    for (int i = 0, n = GetCount(); i < n; i++) {
        SetSelectedListviewItem(GetHandle(), i, bSelected);
    }
}

/// SĂ̑I𔽓]
void CListview::ReverseSelectAll() {
    for (int i = 0, n = GetCount(); i < n; i++) {
        SetSelectedListviewItem(GetHandle(), i, !IsSelectedListviewItem(GetHandle(), i));
    }
}

void CListview::ScrollToSelect() {
    ListView_EnsureVisible(GetHandle(), GetSelected(), FALSE);
}

/// IĂƂ炢ɂȂ悤ɃXN[
void CListview::ScrollToSelectCenter() {
    int nTopIndex = ListView_GetTopIndex(GetHandle()); // Ăԏ̂Ƃ̃CfbNX
    int nCountPerPage = ListView_GetCountPerPage(GetHandle()); // Py[Wӂ̍ڐ
    int nTopMargin = nCountPerPage / 4;
    int nBottomMargin = (nCountPerPage * 3 + 1) / 4;
    int nIndex = GetSelected();
    if (nTopIndex + nTopMargin < nIndex) {
        ListView_EnsureVisible(GetHandle(), nIndex + nBottomMargin, FALSE);
    } else {
        ListView_EnsureVisible(GetHandle(), nIndex - nTopMargin, FALSE);
    }
}

void CListview::SetCheck(int nIndex, bool bCheck) {
    ListView_SetCheckState(GetHandle(), nIndex, bCheck ? TRUE : FALSE);
}

bool CListview::GetCheck(int nIndex) const {
    return ListView_GetCheckState(GetHandle(), nIndex) != FALSE;
}

//------------------------------------------------------------------//

CRadios& CRadios::operator/(int nId) {
    m_vctRadios.push_back(CWnd(m_hDlg).GetCtrl(nId));
    return *this;
}

int CRadios::Get() const {
    for (size_t i = 0, n = m_vctRadios.size(); i < n; i++) {
        if (m_vctRadios[i].GetCheck()) {
            return static_cast<int>(i);
        }
    }
 // Themis_DebugLog(L"WI{^ЂƂ`FbNĂȂۂ");
    return m_nDefault;
}

void CRadios::Set(int m) {
    // xSfalseɂĂwԖڂ̂trueɁB
    // Ăяo邪A̕mۂȂCȂłȂB
    for (size_t i = 0, n = m_vctRadios.size(); i < n; i++) {
        m_vctRadios[i].SetCheck(false);
    }
    m_vctRadios[m].SetCheck(true);
}

}
