#pragma once
#include <boost/iterator/iterator_facade.hpp>
namespace Themis {

/**
    PNXgB

    std::set݂insert/eraseƂgꍇŁAvfȂꍇɂ悳B
*/
template<class T> class single_list : private boost::noncopyable {
private:
    typedef typename boost::call_traits<T>::param_type  param_t;
    typedef typename boost::call_traits<T>::reference ref_t;
    typedef typename boost::call_traits<T>::const_reference cref_t;

    struct SLink {
        SLink*  pNext;
        T       oValue;
        SLink(SLink* p, param_t t): pNext(p), oValue(t) {}
        ~SLink() { delete pNext; }
    };

public:
    single_list(): m_pLink(NULL) {}
    ~single_list() { delete m_pLink; }

    single_list(const single_list<T>& r) {
        inner_assign(r);
    }
    template<class U> single_list& operator=(const single_list<U>& r) {
        delete m_pLink; inner_assign(r); return *this;
    }

    /// iterator
    class iterator: public boost::iterator_facade<typename single_list<T>::iterator, T, boost::forward_traversal_tag, typename single_list<T>::ref_t> {
        friend class single_list<T>;
        friend class boost::iterator_core_access;
        SLink*  m_p;
        iterator(SLink* p): m_p(p) {}
        typename single_list<T>::ref_t dereference() const { return m_p->oValue; }
        void increment() { m_p = m_p->pNext; }
        bool equal(const iterator& r) const { return m_p == r.m_p; }
    public:
        iterator(const iterator& r): m_p(r.m_p) {}
        iterator& operator=(const iterator& r) { m_p = r.m_p; return *this; }
    };
    /// const_iterator
    class const_iterator: public boost::iterator_facade<typename single_list<T>::const_iterator, const T, boost::forward_traversal_tag, typename single_list<T>::cref_t> {
        friend class single_list<T>;
        friend class boost::iterator_core_access;
        const_iterator(SLink* p): m_p(p) {}
        typename single_list<T>::cref_t dereference() const { return m_p->oValue; }
        void increment() { m_p = m_p->pNext; }
        bool equal(const const_iterator& it) const { return m_p == it.m_p; }
        SLink*  m_p;
    public:
        const_iterator(const iterator& r): m_p(r.m_p) {}
        const_iterator(const const_iterator& r): m_p(r.m_p) {}
        const_iterator& operator=(const iterator& r) { m_p = r.m_p; return *this; }
        const_iterator& operator=(const const_iterator& r) { m_p = r.m_p; return *this; }
    };
    iterator begin() { return m_pLink; }
    iterator end() { return NULL; }
    const_iterator begin() const { return m_pLink; }
    const_iterator end() const { return NULL; }

    bool empty() const { return m_pLink == NULL; }

    T& front() const { return m_pLink->oValue; }

    void insert(param_t t) {
        m_pLink = new SLink(m_pLink, t);
    }

    void erase(param_t t) {
        if (m_pLink == NULL) {
        } else if (t == m_pLink->oValue) {
            inner_delete(m_pLink);
        } else {
            SLink* p = m_pLink;
            while (p->pNext != NULL) {
                if (t == p->pNext->oValue) {
                    inner_delete(p->pNext);
                } else {
                    p = p->pNext;
                }
            }
        }
    }
    void erase(const iterator& it) {
        if (it == end()) {
        } else if (it.m_p == m_pLink) {
            inner_delete(m_pLink);
        } else {
            SLink* p = m_pLink;
            while (p->pNext != it.m_p) {
                p = p->pNext;
                Themis_DebugAssert(p != NULL);
            }
            inner_delete(p->pNext);
        }
    }
    void erase(const iterator& it1, const iterator& it2) {
        if (it1 == end() || it1 == it2) {
        } else if (m_pLink == it.m_p) {
            SLink* p2 = it1.m_p->pNext; //it2̈OT
            while (p2->pNext != it2.m_p) {
                p2 = p2->pNext;
            }
            p2->pNext = NULL;
            delete m_pLink;
            m_pLink = it2.m_p;
        } else {
            SLink* p1 = m_pLink->pNext; //it1̈OT
            while (p1->pNext != it1.m_p) {
                p1 = p1->pNext;
            }
            SLink* p2 = it1.m_p->pNext; //it2̈OT
            while (p2->pNext != it2.m_p) {
                p2 = p2->pNext;
            }
            p2->pNext = NULL;
            delete p1->pNext;       //p1̎ == it1p2 == it2܂ł폜
            p1->pNext = it2.m_p;
        }
    }

private:
    template<class U> void inner_assign(const single_list<U>& r) {
        if (!r.empty()) {
            const_iterator it = r.begin();
            SLink* p = m_pLink = new SLink(NULL, *it);
            ++it;
            for (; it != r.end(); ++it) {
                p = p->pNext = new SLink(NULL, *it);
            }
        }
    }

    void inner_delete(SLink*& p) {
        SLink* p2 = p;
        p = p->pNext;       //pP֐i߂
        p2->pNext = NULL;
        delete p2;
    }

private:
    SLink*  m_pLink;
};

}
